from Text2Viz import Text2Viz, require_input_from_user

if __name__ == '__main__':
    visualizations_available = ['ann_text', 'drs', 'graph']

    visualization_required = input(f"Select the desired Visualization: {visualizations_available}\n")

    if visualization_required in visualizations_available:

        [language, text] = require_input_from_user()

        text2viz = Text2Viz('en', '', '2021-12-31', visualization_required)

        if language == 'pt':
            text2viz.narrative.lang = 'pt'
        elif language == 'en':
            text2viz.narrative.lang = 'en'

        text2viz.narrative.text = text

        text2viz.run()
    else:
        print("Visualization not valid")



import os
import sys
import xml.etree.ElementTree as ET
import argparse
import re
import string

from text2story.readers.read import Read
from text2story.readers.token_corpus import TokenCorpus

from text2story.core.narrative import Narrative
from text2story.core.entity_structures import ActorEntity, EventEntity, TimeEntity
from text2story.readers.read_brat import ReadBrat

def dir_path(path):
    if os.path.isdir(path):
        return path
    else:
        raise argparse.ArgumentTypeError(f"readable_dir:{path} is not a valid path")

class ReadECB(Read):
    def __init__(self) -> None:
        self.root = None
        self.tokens = []
        self.actors = []
        self.events = []
        self.times = []
        self.srlinks = []

        # given an id of a sentence, maps to the entities (event, actors, times)
        # of that sentence
        self.sentences_entities = {} 

        self.narrative = Narrative("en", "", "2021-08-08")
        self.read_brat = ReadBrat()

        self.idx_actor = 0
        self.idx_event = 0
        self.idx_time = 0

    def process(self, data_dir):
        return self.read_brat.process(data_dir)

    def process_file(self, data_file):
        return self.read_brat.process_file(data_file)

    def convert_xml_to_brat(self, xml_path, data_dir):
        files_list = []

        for dir_path, dirs, files in os.walk(xml_path):
            for f in files:
                files_list.append({
                    'file': f.replace('.xml', ''),
                    'path': dir_path
                })

        for index, file_dict in enumerate(files_list):
            print(f'Processing file {file_dict["file"]} No: {index + 1} out of {len(files_list)}')
            self.generate_ann_txt_file(file_dict['path'], file_dict['file'], data_dir)

    def generate_ann_txt_file(self, file_path, file_name, data_dir):
        tree = ET.parse(f'{file_path}/{file_name}.xml')
        self.root = tree.getroot()

        txt_file = open(f'{data_dir}/{file_name}.txt', 'w')
        ann_file = open(f'{data_dir}/{file_name}.ann', 'w')

        self.erase_data_structures()

        narrative_text = self.reconstruct_narrative(txt_file)
        self.findall_actors(narrative_text)
        self.findall_events(narrative_text)
        self.findall_time(narrative_text)

        self.findall_relations(narrative_text)

        self.fill_narrative()
        self.construct_ann(self.narrative.ISO_annotation(), ann_file)

    def reconstruct_narrative(self, txt_file):


        for token in self.root.iter('token'):
            self.tokens.append(TokenCorpus(token.text.strip(), token.attrib['t_id'], token.attrib['sentence']))

        current_sentence = 0
        char_counter = 0
        string_to_file = ""

        for token in self.tokens:

            if int(token.sentence) > current_sentence:
                current_sentence = current_sentence + 1
                string_to_file += "\n"

            if token.text != ";" and token.text != "," and token.text != "'" and token.text != ".":
                string_to_file += ' '
                char_counter += 1

            string_to_file += token.text
            # TODO: I dont understand why we need +1 but we need it
            token.offset = char_counter + 1
            char_counter += len(token.text)


        txt_file.write(string_to_file)
        txt_file.flush()
        txt_file.close()

        return string_to_file

    def findall_actors(self, narrative_text):

        type_actor = [("HUMAN_PART_PER","Noun","Per"),\
                ("HUMAN_PART_ORG","Noun","Org"),\
                ("HUMAN_PART_GPE","Noun","Loc"),\
                ("HUMAN_PART_VEH","Noun","Obj"),\
                ("NON_HUMAN_PART","Noun","Obj"),\
                ("NON_HUMAN_PART_GENERIC","Noun","Obj"),\
                ("HUMAN_PART_FAC","Noun","Other"),\
                ("HUMAN_PART_MET","Noun","Other")]

        for atype, apos, aclass in type_actor:
            self.idx_actor = 0
            self.findall_actor_type(narrative_text, atype, apos, aclass)

    def findall_actor_type(self, narrative_text, atype, apos, aclass):
        actor_lst = []
        for actor in self.root.iter(atype):

            actor_token_index_list = []

            for anchor in actor.findall('token_anchor'):
                t_id = anchor.attrib['t_id']
                actor_token_index_list.append(int(t_id))

            if len(actor_token_index_list) > 0:
                actor_lst.append(actor_token_index_list)

        actor_lst.sort(key=lambda x: int(x[0]))
        #import pdb
        #pdb.set_trace()

        for actor_token_index_list in actor_lst:
            self.actors.append(self.process_actor_token_list(actor_token_index_list, apos, aclass, narrative_text))


    def tokenlist2text(self, token_lst, idx_start, text):
        """
        Given a list of tokens, find the string of these tokens in the text parameter.
        """
        if len(token_lst) > 0:
            white_space_re = re.compile("\s")
            fst_token = token_lst[0].text

            # it going to start the search by the first token
            offset = text.find(fst_token, idx_start)
            idx_text = offset + len(fst_token)

            idx_tok = 1

            while idx_tok < len(token_lst):
                current_token = token_lst[idx_tok].text

                # ignore punctuation and every kind of space (tab, new line, etc)
                while idx_text < len(text):
                    m = white_space_re.search(text[idx_text])
                    if m is None and text[idx_text] not in string.punctuation:
                        break
                    idx_text = idx_text + 1

                j = 0    
                # ignore punctuation and every kind of space (tab, new line, etc)
                while j < len(current_token):
                    m = white_space_re.search(current_token)
                    if m is None and current_token not in string.punctuation:
                        break
                    j = j + 1
 
                k = idx_text
                # compare the current token to the current token in the text
                while j < len(current_token) and current_token[j] == text[k]:
                    j = j + 1
                    k = k + 1

                # if there is no match between one of the tokens in the token
                # list, return None
                if j != len(current_token):
                    return
                else:
                    idx_text = k


                idx_tok = idx_tok + 1

            return text[offset:idx_text]
 
    def findall_relations(self, narrative_text):

        if len(self.sentences_entities) == 0:
            print("Warning: There is no entity found to build relations.")
            return
        else:
            for sent_id in self.sentences_entities:
                if "actor" in self.sentences_entities[sent_id]:
                    actor_lst = self.sentences_entities[sent_id]["actor"]
                    event_lst = self.sentences_entities[sent_id]["event"]

                    actor_txt = ""
                    for a in actor_lst:
                        actor_txt = actor_txt + " " + a.text

                    event_txt = ""
                    for a in event_lst:
                        event_txt = event_txt + " " + a.text

                    print("--- SENT %s ---" % sent_id)
                    print("ACTORS %s" % actor_txt)
                    print("EVENT %s" % event_txt)
                    print()



    def process_actor_token_list(self, index_list, lexical_head, actor_type, narrative_text):
        raw_str = ""
        tok_lst = []

        for index in index_list:

            # collecting entities per sentence in order to 
            # detect relation between them
            sent_id = self.tokens[index - 1].sentence
            if sent_id in self.sentences_entities:
                if "actor" in self.sentences_entities[sent_id]:
                    self.sentences_entities[sent_id]["actor"].append(self.tokens[index - 1])
                else:
                    self.sentences_entities[sent_id]["actor"] = [self.tokens[index - 1]]
            else:
                self.sentences_entities[sent_id] = {"actor":[self.tokens[index - 1]]}

            if self.tokens[index - 1].text in ".,;?!":
                raw_str = raw_str.rstrip()
            raw_str += self.tokens[index - 1].text + " "
            tok_lst.append(self.tokens[index - 1])

        raw_str = raw_str.rstrip()


        offset_start = narrative_text.find(raw_str, self.idx_actor)
        if offset_start < 0:
            token_txt = self.tokenlist2text(tok_lst, self.idx_actor, narrative_text)
            if token_txt is not None:
                offset_start = narrative_text.find(token_txt, self.idx_actor)


        self.idx_actor = offset_start + len(raw_str)
        #return ActorEntity(raw_str, (
        #    self.tokens[index_list[0] - 1].offset, self.tokens[index_list[0] - 1].offset + len(raw_str)),
        #                   lexical_head,
        #                   actor_type)
        return ActorEntity(raw_str, (
                           offset_start, offset_start + len(raw_str)),
                           lexical_head,
                           actor_type)

    def findall_events(self, narrative_text):

        type_events = [('ACTION_OCCURRENCE',"Occurrence","Pos"),\
                ('NON_ACTION_OCCURRENCE',"Occurrence","Neg"),\
                ('ACTION_PERCEPTION','Perception','Pos'),\
                ('NON_ACTION_PERCEPTION','Perception','Neg'),\
                ('ACTION_REPORTING',"Reporting","Pos"),\
                ('NON_ACTION_REPORTING','Reporting','Neg'),\
                ('ACTION_ASPECTUAL','Aspectual','Pos'),\
                ('NON_ACTION_ASPECTUAL','Aspectual','Neg'),\
                ('ACTION_STATE','State','Pos'),\
                ('NON_ACTION_STATE','State','Neg'),\
                ('ACTION_CAUSATIVE','Causative','Pos'),\
                ('NON_ACTION_CAUSATIVE','Causative','Neg'),\
                ('ACTION_GENERIC','Generic','Pos'),\
                ('NON_ACTION_GENERIC','Generic','Neg')]

        for etype, eclass, polarity in type_events:
            self.idx_event = 0
            self.findall_event_type(narrative_text, etype, eclass, polarity)

    def findall_event_type(self, narrative_text, etype, eclass, polarity):

        event_action_lst = []
        for event in self.root.iter(etype):

            event_token_index_list = []

            for anchor in event.findall('token_anchor'):
                t_id = anchor.attrib['t_id']
                event_token_index_list.append(int(t_id))

            if len(event_token_index_list) > 0:
                event_action_lst.append(event_token_index_list)

        event_action_lst.sort(key=lambda x:x[0])
        for event_token_index_list in event_action_lst:
            event_obj = self.process_event_token_list(event_token_index_list, eclass, polarity, narrative_text)
            # this only happens when there are two events with the same token
            # so to avoid it, ignore one of the events
            if event_obj.character_span[0] > 0:
                self.events.append(event_obj)


    def process_event_token_list(self, index_list, event_class, polarity, narrative_text):

        # TODO: create a solution when the event has not adjacent tokens
        raw_str = ""
        tok_lst = []
        for index in range(index_list[0], index_list[-1] + 1):

            # collecting entities per sentence in order to 
            # detect relation between them
            sent_id = self.tokens[index - 1].sentence
            if sent_id in self.sentences_entities:
                if "event" in self.sentences_entities[sent_id]:
                    self.sentences_entities[sent_id]["event"].append(self.tokens[index - 1])
                else:
                    self.sentences_entities[sent_id]["event"] = [self.tokens[index - 1]]
            else:
                self.sentences_entities[sent_id] = {"event":[self.tokens[index - 1]]}

            if self.tokens[index - 1].text in ".,;?!":
                raw_str = raw_str.rstrip()
            raw_str += self.tokens[index - 1].text + " "
            tok_lst.append(self.tokens[index - 1])

        raw_str = raw_str.rstrip()

        offset_start = narrative_text.find(raw_str, self.idx_event)
        if offset_start < 0:
            token_txt = self.tokenlist2text(tok_lst, self.idx_event, narrative_text)
            if token_txt is not None:
                offset_start = narrative_text.find(token_txt, self.idx_event)

        self.idx_event = offset_start + len(raw_str)

        return EventEntity(raw_str,
                           (offset_start,
                            offset_start + len(raw_str)),
                           event_class, polarity)

    def findall_time(self, narrative_text):
        time_types = [("TIME_DATE","Date","Date"),\
                ("TIME_THE_DAY","Time","Time"),\
                ("TIME_DURATION","Duration","Duration"),\
                ("TIME_REPETITION","Time","Set")]

        for type_t, value, timex in time_types:
            self.idx_time = 0
            self.findall_time_type(type_t, value, timex, narrative_text)

    def findall_time_type(self,time_t, value, timex, narrative_text):

        time_lst = []
        for time in self.root.iter(time_t):

            time_token_index_list = []

            for anchor in time.findall('token_anchor'):
                t_id = anchor.attrib['t_id']
                time_token_index_list.append(int(t_id))

            if len(time_token_index_list) > 0:
                time_lst.append(time_token_index_list)

        time_lst.sort(key=lambda x:x[0])

        for time_token_index_list in time_lst:
            self.times.append(self.process_time_token_list(time_token_index_list, value, timex, narrative_text))

    def process_time_token_list(self, index_list, value, timex_type, narrative_text):
        raw_str = ""

        # SOmetimes the tokens are not connected by space. Usually, 
        # this happens with punctuation. Then, it checks if the token is 
        # a punctuation symbol, then remove the space before the punctuation.

        for index in range(index_list[0], index_list[-1] + 1):

            # collecting entities per sentence in order to 
            # detect relation between them
            sent_id = self.tokens[index - 1].sentence
            if sent_id in self.sentences_entities:
                if "time" in self.sentences_entities[sent_id]:
                    self.sentences_entities[sent_id]["time"].append(self.tokens[index - 1])
                else:
                    self.sentences_entities[sent_id]["time"] = [self.tokens[index - 1]]
            else:
                self.sentences_entities[sent_id] = {"time":[self.tokens[index - 1]]}

            if self.tokens[index - 1].text in ".,;?!":
                raw_str = raw_str.rstrip()
            raw_str += self.tokens[index - 1].text + " "

        raw_str = raw_str.rstrip()

        offset_start = narrative_text.find(raw_str, self.idx_time)
        self.idx_time = offset_start + len(raw_str)

        return TimeEntity(raw_str,
                (offset_start, offset_start + len(raw_str)), value,
                          timex_type)

    def fill_narrative(self):
        actors_dict = {}
        events_dict = {}
        times_dict = {}

        index = 0

        for actor in self.actors:
            actors_dict['T' + str(index)] = actor
            index = index + 1

        for event in self.events:
            events_dict['T' + str(index)] = event
            index = index + 1

        for time in self.times:
            times_dict['T' + str(index)] = time
            index = index + 1

        self.narrative.actors = actors_dict
        self.narrative.events = events_dict
        self.narrative.times = times_dict

    def construct_ann(self, iso_annotation, ann_file):
        ann_file.write(iso_annotation)
        ann_file.flush()
        ann_file.close()

    def to_column(self, data_dir, output_dir):
        return self.read_brat.toColumn(data_dir, output_dir)

    def erase_data_structures(self):
        self.tokens = []
        self.actors = []
        self.events = []
        self.times = []
        self.narrative = Narrative("en", "", "2021-08-08")


def main(xml_dir: str = None, data_dir: str = None, output_dir: str = None):

    r = ReadECB()
    r.convert_xml_to_brat(xml_dir, data_dir)
    # r.to_column(data_dir, output_dir)


if __name__ == '__main__':
    my_parser = argparse.ArgumentParser(description='Read ECB+ corpus')

    my_parser.add_argument("xmldir", action='store', type=dir_path, help="The directory that contains the xml file (ECB+ format).")
    my_parser.add_argument("datadir", action='store', type=dir_path, help="The directory where the brat files will be stored.")

    args = my_parser.parse_args()
    main(args.xmldir, args.datadir)





import os
import re

DRS_TAGS = {'events': '» EVENTS',
            'actors': '» ACTORS',
            'relations': '» RELATIONS'}


def natural_sort(lst):
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return sorted(lst, key=alphanum_key)


def remove_quotes(text):
    return text.replace('"', '')


def get_drs_files(drs_dir='../drs_files'):
    return natural_sort([os.path.join(drs_dir, f) for f in os.listdir(drs_dir) if
                         (os.path.isfile(os.path.join(drs_dir, f)) and '_drs.txt' in f)])


def get_drs_str(drs_file):
    with open(drs_file, 'r') as drs_file:
        drs = drs_file.read()

    # Remove # from drs
    drs = drs.replace('# ', '')

    return drs


def get_entities(drs):
    events = drs.split(DRS_TAGS['events'])[1].split(DRS_TAGS['actors'])[0]
    actors = drs.split(DRS_TAGS['actors'])[1].split(DRS_TAGS['relations'])[0]
    relations = drs.split(DRS_TAGS['actors'])[1].split(DRS_TAGS['relations'])[1]

    return events, actors, relations


# EVENTS

def get_events_list(events):
    events_list = events.split('\n\n')

    return events_list


def get_events_dict(events_list):
    # Refactor this v
    '''
    returns a dictionary {event_tag: event_type}
    '''
    arrow_pattern = ' -> '
    events_dict_aux = {}

    for event in events_list:
        event_split = event.split(arrow_pattern)

        e = event_split[0]

        e_label = e[e.find("(") + 1:e.find(")")]
        e = e.replace(e_label, '').replace(' ()', '').replace('\n', '')

        if not e:
            continue

        events_dict_aux[e] = e_label

    # Ensure that events are ordered by their tags
    events_dict = {}
    for event_tag in natural_sort(events_dict_aux.keys()):
        events_dict[event_tag] = events_dict_aux[event_tag]

    return events_dict


# ACTORS

def get_actors_list(actors):
    actors = os.linesep.join([s for s in actors.splitlines() if s])
    actors_list = actors.splitlines()
    return actors_list


def get_actors_dict(actors_list):
    '''
    returns a dictionary {actor_tag: actor_name}
    '''
    arrow_pattern = ' -> '
    actors_dict_aux = {}

    for actor in actors_list:
        actor_split = actor.split(arrow_pattern)
        actors_dict_aux[actor_split[0]] = actor_split[1]

    # Ensure that actors are ordered by their tags
    actors_dict = {}
    for actor_tag in natural_sort(actors_dict_aux.keys()):
        actors_dict[actor_tag] = actors_dict_aux[actor_tag]

    return actors_dict


# RELATIONS

def get_relations_list(relations):
    relations = os.linesep.join([s for s in relations.splitlines() if s])
    relations_list = relations.splitlines()
    return relations_list


def get_relations_triplets(relations_list):
    '''
    returns a triplet (actor1, relation, actor2)
    '''
    dash_pattern = ' - '

    relations_triplets = []
    for relation in relations_list:
        relation_split = relation.split(dash_pattern)
        relation_triplet = (
            relation_split[0], relation_split[1], relation_split[2])
        relations_triplets.append(relation_triplet)

    return relations_triplets


def get_relations_per_events(relations_triplets, actors_dict, events_dict):
    # dict {'Ex': [('Ty', 'type'), ...]}
    relations_per_events = {}

    for r in relations_triplets:
        # Get only relations between actors and events
        if r[0] in actors_dict and r[2] in events_dict:
            if r[2] in relations_per_events:
                relations_per_events[r[2]].append((r[0], r[1]))
            else:
                relations_per_events[r[2]] = [(r[0], r[1])]
        elif r[0] in events_dict and r[2] in actors_dict:
            if r[0] in relations_per_events:
                relations_per_events[r[0]].append((r[2], r[1]))
            else:
                relations_per_events[r[0]] = [(r[2], r[1])]

    return relations_per_events


def explode_relations_per_events(relations_per_events):
    # dict {'Ex': [('Ty', 'type'), ...]}
    relations_per_events_exploded = {}
    seen = []
    for ev, rels in relations_per_events.items():
        for rel in rels:
            for rel_aux in rels:
                # If iterating over same event relation and if there is more than one relation, skip
                if (rel == rel_aux and len(rels) > 1):
                    continue

                dummy_triplet = (rel[0], ev, rel_aux[0])

                exists = False
                for r in seen:
                    # If triplet exists in events_relations, do not append
                    triplet_to_check = (r[0], r[1], r[2])
                    if sorted(triplet_to_check) == sorted(dummy_triplet):
                        exists = True
                        break

                if not exists:
                    relations_per_events_exploded.setdefault(ev, []).append(
                        (rel[0], rel[1] + '/' + rel_aux[1], rel_aux[0]))
                    seen.append(dummy_triplet)

    return relations_per_events_exploded


def get_non_event_relations(relations_triplets, events_relations, actors_dict, events_dict):
    non_event_relations = []
    for r in relations_triplets:
        # Get only opposite of relations between actors and events (relations between actors)
        # Written this way to match other function
        if not ((r[0] in actors_dict and r[2] in events_dict) or (r[0] in events_dict and r[2] in actors_dict)):
            # Ignore pronominal relations as they do not introduce useful information
            if r[1] != 'pronominal':
                # Ignore relations between events
                if r[0] not in events_dict and r[2] not in events_dict:
                    non_event_relations.append(r)

    return non_event_relations


def get_redundant_dict(relations_triplet, events_dict):
    '''
    returns a dict
    whose keys are a actors, and the values are lists of equivalent actors (sameHead, objIdentity, synonymy)
    eg: {'actor_x: ['actor_y', 'actor_z']}
    '''

    redundant_dict_aux = {}

    for rel in relations_triplet:
        source = rel[0]
        r = rel[1]
        target = rel[2]

        # Ignore if self relation
        if source == target:
            continue

        # Ignore if relation includes event
        if source in events_dict or target in events_dict:
            continue

        # Ignore if relation does not imply redundancy
        if r != 'sameHead' and r != 'objIdentity' and r != 'synonymy':
            continue

        if target in redundant_dict_aux:
            if source not in redundant_dict_aux[target]:
                redundant_dict_aux[target].append(source)
        else:
            redundant_dict_aux[target] = [source]

    # Simplify
    for tag in list(redundant_dict_aux.keys()):
        # Gets keys from dict in which tag appears as value of those keys
        key_in_value = [key for key, value in redundant_dict_aux.items() if tag in value]

        for k in key_in_value:
            redundant_dict_aux[k].extend(redundant_dict_aux[tag])

        # If tag was a redundant actor, remove it from dict
        if len(key_in_value) > 0:
            del redundant_dict_aux[tag]

    # Order actors by their tags
    redundant_dict = {}
    for actor_tag in natural_sort(redundant_dict_aux.keys()):
        redundant_dict[actor_tag] = natural_sort(
            sorted(set(redundant_dict_aux[actor_tag]), key=redundant_dict_aux[actor_tag].index))

    return redundant_dict


def resolve_redundancy(redundant_dict, actors_dict, relations_triplets):
    '''
    change actors_dict and relations_triplets by removing redundant actors
    '''
    for actor_1, redund_list in redundant_dict.items():

        for actor_aux in redund_list:
            # Add redundant actor to original actor in actor_dict
            actors_dict[actor_1] = actors_dict[actor_1] + ', ' + actors_dict[actor_aux]

            # Change reference from actor_aux to actor_1 in relations
            for i, rel in enumerate(relations_triplets):
                if rel[0] == actor_aux:
                    rel = list(rel)
                    rel[0] = actor_1
                    relations_triplets[i] = tuple(rel)
                if rel[2] == actor_aux:
                    rel = list(rel)
                    rel[2] = actor_1
                    relations_triplets[i] = tuple(rel)

    # Delete redundant actors from actors_dict
    for act, redund_list in redundant_dict.items():
        for redund_actor in redund_list:
            try:
                del actors_dict[redund_actor]
            except KeyError as ke:
                # print('Actor ' + str(ke) + ' already removed')
                pass

    relations_triplets = [rel for rel in relations_triplets if not (
            (rel[0] == rel[2]) and (rel[1] == 'sameHead' or rel[1] == 'synonymy' or rel[1] == 'objIdentity'))]

    return actors_dict, relations_triplets


def create_mscgen(actors_dict, events_dict, events_relations, non_event_relations):
    # Begin mscgen string
    msc_string = 'msc {\nwordwraparcs=true, hscale="1.8";\n'

    # Build actors mscgen
    for i, (tag, actor) in enumerate(actors_dict.items()):
        msc_string += tag + ' [label="' + remove_quotes(actor) + '"]'
        if i < len(actors_dict) - 1:
            msc_string += ', '
        else:
            msc_string += ';\n\n'

    # Build relations mscgen

    # Events relations
    for ev, rels in events_relations.items():

        for i, rel_triplet in enumerate(rels):
            msc_string += rel_triplet[0] + '=>' + rel_triplet[2] + ' [label="' + remove_quotes(events_dict[ev]) + ' (' + \
                          rel_triplet[1] + ')' + '"];'
            # if i == 0:
            #     msc_string += ' [label="' + remove_quotes(events_dict[ev]) + '(' + rel_triplet[1] + ')' + '"]'
            # else:
            #     msc_string += ' [label="' + rel_triplet[1] + '"]'

        #     if i < len(rels)-1:
        #         msc_string += ',\n'

        # msc_string += ';\n'

    # Non events relations
    for rel_triplet in non_event_relations:
        msc_string += rel_triplet[0] + '=>' + rel_triplet[2] + \
                      ' [label="' + rel_triplet[1] + '"];\n'

    # Close mscgen string
    msc_string += '}'

    return msc_string


def print_dict(data):
    for k, v in data.items():
        print(k + ': ' + str(v))
    print()


def print_list(data):
    for elem in data:
        print(elem)
    print()


def parse_drs(drs_file, debug=False):
    try:
        drs = get_drs_str(drs_file)
        print('Parsing file: ' + drs_file + '\n')
    except FileNotFoundError as e:
        print(e)
        return 'msc {}'

    events, actors, relations = get_entities(drs)

    # Events
    events_list = get_events_list(events)
    events_dict = get_events_dict(events_list)

    # Actors
    actors_list = get_actors_list(actors)
    actors_dict = get_actors_dict(actors_list)

    # Relations
    relations_list = get_relations_list(relations)
    relations_triplets = get_relations_triplets(relations_list)

    if debug:
        print('===== ORIGINAL ELEMENTS FROM DRS ===== \n')

        print('EVENTS: ')
        print_dict(events_dict)

        print('ACTORS: ')
        print_dict(actors_dict)

        print('RELATIONS: ')
        print_list(relations_triplets)

    # Resolve actors redundancy
    redundant_dict = get_redundant_dict(relations_triplets, events_dict)
    actors_dict, relations_triplets = resolve_redundancy(redundant_dict, actors_dict, relations_triplets)


    # Resolve chain relations
    relations_per_events = get_relations_per_events(relations_triplets, actors_dict, events_dict)
    events_relations = explode_relations_per_events(relations_per_events)

    non_event_relations = get_non_event_relations(relations_triplets, events_relations, actors_dict, events_dict)

    if debug:
        print('===== PROCESSING ===== \n')

        print('REDUNDANT ACTORS: ')
        print_dict(redundant_dict)

        print('ACTORS: ')
        print_dict(actors_dict)

        print('RELATIONS: ')
        print_list(relations_triplets)

        print('RELATIONS PER EVENT: ')
        print_dict(events_relations)

        print('NON EVENT RELATIONS: ')
        print_list(non_event_relations)

    print('===== ELEMENTS TO DISPLAY ===== \n')

    print('ACTORS: ')
    print_dict(actors_dict)

    print('EVENTS: ')
    print_dict(events_dict)

    print('RELATIONS PER EVENTS: ')
    print_dict(events_relations)

    print('NON EVENT RELATIONS: ')
    print_list(non_event_relations)

    return actors_dict, events_dict, events_relations, non_event_relations


def get_msc_data(drs_file, debug=False):
    actors_dict, events_dict, events_relations, non_event_relations = parse_drs(drs_file)

    msc_string = create_mscgen(actors_dict, events_dict, events_relations, non_event_relations)

    print('MSCGEN STRING: ')
    print(msc_string)
    print()

    return msc_string


def get_graph_data(drs_file, debug=False):
    actors_dict, events_dict, events_relations, non_event_relations = parse_drs(drs_file)

    # Events relations
    event_relations = []
    for ev, rels in events_relations.items():
        for i, rel_triplet in enumerate(rels):
            event_relations.append(
                (rel_triplet[0], remove_quotes(events_dict[ev]) + ' (' + rel_triplet[1] + ')', rel_triplet[2]))

    return actors_dict, non_event_relations, event_relations



from text2story.select.bubble import BubbleMap, BigBubble, Bubble 
from text2story.readers.read_brat import ReadBrat
import pdflatex as ptex
from pdf2image import convert_from_path

import os
import re
import sys

tikz_doc = "\\documentclass{standalone}\n\\usepackage{pgf,tikz}\n\\usetikzlibrary{arrows}\n\\usepackage{listofitems} % for \\readlist to create arrays"

def bubble_header():
    return '''
\\usepackage[T1]{fontenc}
\\usetikzlibrary{calc}
\\usetikzlibrary{arrows.meta}
\\usepackage{relsize}

    % Create style for node circles
	           \\tikzstyle{state}=[circle,
                   minimum size =13cm,
                   draw=white,
                   thick
                ]

               \\tikzstyle{report}=[
                  circle,
                  minimum size =5cm,
                  draw=white,
                  fill=orange!80,
                  thick
                ]

                \\tikzstyle{agent}=[
                  rectangle,
                  draw=black,
                  fill=teal!80,
                  rounded corners=1pt,
                  thick
                ]

                \\tikzstyle{event}=[
                  circle,
                  minimum size =1cm,
                  draw=black,
                  fill=white,
                node distance=\\nodeDist,
                thick
              ]

    

             \\begin{document}

             \\begin{tikzpicture}


             % esse sera o container
            \\node[state] (A) at (0,0) {};'''

def compute_angles(nevents):
    if nevents < 4:
        count_angle = 90
    else:
        count_angle = 360 // nevents
    fst_angle = count_angle
    angle_lst = []

    while fst_angle >= -360 and len(angle_lst) < nevents:
        angle_lst.append(fst_angle)
        fst_angle = fst_angle - count_angle
    return angle_lst

def start_inside_angle(bubble_idx, nbubbles):

    
    angle_lst = compute_angles(nbubbles)

    bubble_angle = angle_lst[bubble_idx]

    if bubble_angle > 0 and bubble_angle <= 90:
        return 0
    elif bubble_angle > -90 and bubble_angle <= 0:
        return -90
    elif bubble_angle > -180 and bubble_angle <= -90:
        return -180
    else:
        return -270

def draw_big_bubble(big_bubble, type_event,  angle, map_id2nodes):

    sent_id = big_bubble.bubble_.event.sent_id

    tikz_str = ""
    
    event_text = big_bubble.bubble_.get_event_text() 
    id_ann = big_bubble.bubble_.event.id_ann[0]

    sent_id_text = str(float(sent_id))
    whole, frac = sent_id_text.split(".") 

    name_node = "%s%s_%s" % (type_event.lower(), whole, frac[:1])
    map_id2nodes[id_ann] = name_node
    big_bubble.bubble_.name = name_node

    tikz_str += "\\node[report] (%s) at (A.%d) {\\footnotesize %s:%s};\n" % (name_node, angle, sent_id_text, event_text)
    tikz_str += "\n" 

    if big_bubble.bubble_.agent.span != []:
        # draw agents
        shift = 0.5
        current_agent = big_bubble.bubble_.agent
        idx_agent = 0

        while current_agent != None:
            name_agent_node = "agent%s_%s_%d" % (whole, frac[:1], idx_agent)
            current_agent.name = name_agent_node
            tok_txt_agent = [tok.text for tok in current_agent.span]
            agent_txt = " ".join(tok_txt_agent)

            tikz_str += "\\node[agent] (%s) at ([shift=({90:%f cm})]%s) {\\footnotesize %s};\n" % (name_agent_node, shift, name_node, agent_txt)
            tikz_str += "\n" 

            shift += 0.5
            idx_agent += 1
            current_agent = current_agent.next_agent


    return tikz_str

def draw_little_bubbles(big_bubble, map_id2nodes):


    sent_id = big_bubble.bubble_.event.sent_id

    # draw the little bubbles
    little_bubble_lst_str = "\\def\\eventlst%d{{" % (sent_id)

    # sort little bubbles by their offset text position
    big_bubble.sort_by_offset()

    for lbubble in big_bubble.little_bubbles:

        little_bubble_lst_str =  little_bubble_lst_str + "\"%s\"," % lbubble.get_event_text()

    little_bubble_lst_str = little_bubble_lst_str + "}}\n"

    nevents = len(big_bubble.little_bubbles)

    for idx in range(nevents):
        id_ann = big_bubble.little_bubbles[idx].event.id_ann[0]
        name_node = "r%devent%d" % (sent_id,  idx)
        big_bubble.little_bubbles[idx].name = name_node

        map_id2nodes[id_ann] = name_node

    angle_lst = compute_angles(nevents)

    little_bubble_lst_str = little_bubble_lst_str + "\n\\foreach[count=\\i from 0] \\angle in {"

    angle_lst_str = ",".join(map(str,angle_lst))

    little_bubble_lst_str = little_bubble_lst_str + angle_lst_str + "}{\n"
    
    little_bubble_lst_str = little_bubble_lst_str + "\\node[event] (r%devent\\i) at ([shift=({\\angle:2 cm})]%s) {\\scriptsize \\pgfmathparse{\\eventlst%d[\\i]} \\pgfmathresult};}" % (sent_id,big_bubble.bubble_.name,sent_id)
    
    return little_bubble_lst_str

def draw_agent_relation(agent):

    rel_str = "\n"

    for rel_agent in agent.relations_agent:
        name_rel_agent = rel_agent.agent.name
        link_text = rel_agent.rel_type.split("_")[1]

        if rel_agent.out:
            rel_str = rel_str + "\n \\draw [<-, very thick, arrows = {-Latex[length=3mm, width=1.5mm]}] "
        else:
            rel_str = rel_str + "\n \\draw [->, very thick, arrows = {-Latex[length=3mm, width=1.5mm]}] "

        rel_str = rel_str + "(%s) edge [out=90,midway, above] node[sloped, fill=white] {\\tiny %s} (%s);\n" % (agent.name, link_text, rel_agent.agent.name)

    for rel_bubble in agent.relations_bubbles:
        if isinstance(rel_bubble.bubble, BigBubble):
            name_rel_bubble = rel_bubble.bubble.bubble_.name
        else:
            name_rel_bubble = rel_bubble.bubble.name

        link_text = rel_bubble.rel_type.split("_")[1]

        if rel_bubble.out:
            rel_str = rel_str + "\n \\draw [<-, very thick, arrows = {-Latex[length=3mm, width=1.5mm]}] "
        else:
            rel_str = rel_str + "\n \\draw [->, very thick, arrows = {-Latex[length=3mm, width=1.5mm]}] "

        rel_str = rel_str + "(%s) edge [out=90,midway, above, looseness = 3] node[sloped, fill=white] {\\tiny %s} (%s.text);\n" % (agent.name, link_text, name_rel_bubble)

    return rel_str 

def draw_bubble_relation(bubble1, rel, map_id2nodes, inside_angle_map, nevents = 0):

    
    if isinstance(bubble1, BigBubble):
        bubble1_event_id = bubble1.bubble_.event.id_ann[0]
        name_bubble1 = bubble1.bubble_.name
        # it gets the sent_id only if bubble1 is a big_buble
        sent_id = int(re.match('.*?([0-9]+)$', name_bubble1).group(1))

    else:
        bubble1_event_id = bubble1.event.id_ann[0]
        name_bubble1 = bubble1.name
        sent_id = int(re.match("r\d+", name_bubble1).group(0)[1:])

    if bubble1_event_id not in map_id2nodes:
        return

    bubble2 = rel.bubble_pointer

    if isinstance(bubble2, BigBubble):
        name_bubble2 = bubble2.bubble_.name
        bubble2_event_id = bubble2.bubble_.event.id_ann[0]
    else:
        name_bubble2 = bubble2.name
        bubble2_event_id = bubble2.event.id_ann[0]

    if bubble2_event_id not in map_id2nodes:
        return

    rel_str = ""
    

    edge_type = rel.edge_type
    name_edge = edge_type.split("_")[1]

    
    # get the index number (a counter that is the index of the event inside a big bubble)
    bubble_idx = int(re.match('.*?([0-9]+)$', name_bubble2).group(1))

    # bubble2 is in the same sentence of bubble1    
    if name_bubble2.startswith("r%d" % sent_id):

        #print(re.match('.*?([0-9]+)$', name_bubble_pointer).group())


        if name_bubble2 not in inside_angle_map:
            inside_angle_map[name_bubble2] = start_inside_angle(bubble_idx, nevents)

        inside_angle = inside_angle_map[name_bubble2]

        if rel.out: # arg1: bubble2 arg2: bubble1
            rel_str = rel_str + "\n \\draw [<-, dashed, arrows = {-Latex[length=5mm, width=2mm]}] " 
        else:
            rel_str = rel_str + "\n \\draw [->, dashed, arrows = {-Latex[length=5mm, width=2mm]}] "

        if isinstance(bubble1, BigBubble):
            inside_angle1 = 0
            rel_str = rel_str + "(%s.center)" % name_bubble1
        else:
            # TODO: testar com inside angle com o bubble1
            if name_bubble1 not in inside_angle_map:
                # get the index number (a counter that is the index of the event inside a big bubble)
                bubble_idx1 = int(re.match('.*?([0-9]+)$', name_bubble1).group(1))
                inside_angle_map[name_bubble1] = start_inside_angle(bubble_idx1, nevents)

            inside_angle1 = inside_angle_map[name_bubble1]
            inside_angle_map[name_bubble1] -= 60 
            rel_str = rel_str + "(%s)" % name_bubble1


        rel_str = rel_str + " edge [out=%d,in=%d] node[midway, fill=white, sloped] {\\footnotesize %s} (%s);" % (inside_angle1, inside_angle,name_edge,  name_bubble2)

        inside_angle_map[name_bubble2] -= 60
               
    else:
        if rel.out: # arg1:bubble2 arg2:bubble1:
            rel_str = rel_str + "\n \\draw [<-, very thick, arrows = {-Latex[length=5mm, width=2mm]}] "
        else:
            rel_str = rel_str + "\n \\draw [->, very thick, arrows = {-Latex[length=5mm, width=2mm]}] "

        isbubble = re.match("r\d+", name_bubble1)
        if isbubble:

            sent_id1 = isbubble.group() # the sent id, if bubble 1 is not a big bubble
            sent_id2 = re.match("r\d+", name_bubble2) # the sent_id , if bubble 2 is not a big bubble

            if sent_id2:
                sent_id2 = sent_id2.group()

                # two little bubbles inside the same big bubble?
                if sent_id1 == sent_id2:

                    if name_bubble2 not in inside_angle_map:
                        inside_angle_map[name_bubble2] = start_inside_angle(bubble_idx,nevents)
                        
                    inside_angle = inside_angle_map[name_bubble2]
                    rel_str = rel_str + "(%s) edge [out=0,in=%d] node[midway, fill=white, sloped] {\\footnotesize %s} (%s);" % (name_bubble1,inside_angle,name_edge,  name_bubble2)

                    inside_angle_map[name_bubble2] -= 60    

                else:
                    rel_str = rel_str + "(%s) -- (%s) node[midway,fill=white,sloped] {\\footnotesize %s};" % (name_bubble1, name_bubble2, name_edge)
            else:
                rel_str = rel_str + "(%s) -- (%s) node[midway,fill=white,sloped] {\\footnotesize %s};" % (name_bubble1, name_bubble2, name_edge)
        else:

            rel_str = rel_str + "(%s) -- (%s) node[midway,fill=white,sloped] {\\footnotesize %s};" % (name_bubble1, name_bubble2, name_edge)

    
    return rel_str


def build_fig(tok_lst, type_event = "Reporting", type_rel_lst = []):

    tikz_str = tikz_doc + "\n" + bubble_header()
    
    bubble_map = BubbleMap()
    bubble_map.build_map(tok_lst, type_event, type_rel_lst)

    nevents =  len(bubble_map.map)
    angle_lst = compute_angles(nevents)

    map_id2nodes = {} # a map of id_ann to nodes id
    
    for idx, sent_id in enumerate(bubble_map.map.keys()):

        big_bubble = bubble_map.map[sent_id]
        tikz_str = tikz_str + "\n" + draw_big_bubble(big_bubble, type_event, angle_lst[idx], map_id2nodes)    

        tikz_str = tikz_str + "\n" + draw_little_bubbles(big_bubble, map_id2nodes)

    
    tikz_str = tikz_str + "\n"

    
    inside_angle_map = {}
    #print("Big Bubbles: ", len(bubble_map.map))
    for sent_id in bubble_map.map:


        big_bubble = bubble_map.map[sent_id]
        
        bubble_event_id = big_bubble.bubble_.event.id_ann[0]


        if bubble_event_id not in map_id2nodes:
            continue

        nevents = len(big_bubble.little_bubbles)
        # draw agent relations
        if big_bubble.bubble_.agent.span != []:
            current_agent = big_bubble.bubble_.agent

            while (current_agent != None):
                rel_str = draw_agent_relation(current_agent)
                if rel_str is not None:
                    tikz_str += rel_str
                current_agent = current_agent.next_agent

        # draw relations of the big bubble
        for rel in big_bubble.bubble_.relations: 

            #if rel.edge_type.startswith("TLINK") and rel.edge_type != "TLINK_identity":
            if rel.edge_type.startswith("TLINK"):
                continue
            rel_str = draw_bubble_relation(big_bubble, rel, map_id2nodes, inside_angle_map, nevents)  
            if rel_str is not None:
                tikz_str += rel_str

            
            tikz_str = tikz_str + "\n"
    
            # draw relations of the little bubbles
            for little_bubble in big_bubble.little_bubbles:

                lbubble_event_id = little_bubble.event.id_ann[0]
                #print("==>",lbubble_event_id)
                #import pdb
                #pdb.set_trace()

                if lbubble_event_id not in map_id2nodes:
                     continue

                # is it in the same big bubble? adjust the angle inside
                for rel in little_bubble.relations:
                    #if rel.edge_type.startswith("TLINK") and rel.edge_type != "TLINK_identity":
                    if rel.edge_type.startswith("TLINK"):
                        continue
                    rel_str = draw_bubble_relation(little_bubble, rel, map_id2nodes, inside_angle_map, nevents)  
                    if rel_str is not None:
                         tikz_str += rel_str

                

    tikz_str = tikz_str + "\n" + "\\end{tikzpicture}\n\\end{document}\n"

    return tikz_str, bubble_map

def build_fig_ann(ann_file, output_dir):

    reader = ReadBrat()
    data = reader.process_file(ann_file)

    #Serao apenas as relações temporais que nao envolvam reporting events ligado pelo TLINK identity.
    tikz_str, bubble_map = build_fig(data, "Reporting", ["TLINK"])

    bubble_map.to_json("output.txt")

    ann_file_base = os.path.basename(ann_file)

    output_file = os.path.join(output_dir, "%s.tex" % ann_file_base)
    with open(output_file, "w") as fd:
        fd.write(tikz_str)

    pdfl = ptex.PDFLaTeX.from_texfile(output_file)
    #os.system("pdflatex -halt-on-error -output-directory %s %s" % (output_dir, output_file))

    pdf_file = os.path.join(output_dir, "%s.pdf" % ann_file_base)

    pdf, log, completed_process = pdfl.create_pdf()
    with open(pdf_file, 'wb') as pdfout:
        pdfout.write(pdf)

    png_file = os.path.join(output_dir, "%s.png" % ann_file_base)

    pages = convert_from_path(pdf_file, 500)
    for page in pages:
        # usually only one page
        page.save(png_file, 'PNG')

    #os.system("convert  -density 600 %s  %s" % (pdf_file, png_file))
    os.system("rm %s" % (pdf_file))





import os
import pandas as pd
import shutil
from pathlib import Path

import string

# adding root directory
import sys
sys.path.insert(0, '..')
sys.path.insert(0, '../../')

from sklearn.pipeline import FeatureUnion
from sklearn.pipeline import Pipeline
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.preprocessing import OneHotEncoder
from sklearn.compose import ColumnTransformer
from sklearn.linear_model import LogisticRegression
from sklearn import metrics

from sklearn_crfsuite.utils import flatten
import sklearn_crfsuite
from sklearn_crfsuite import scorers
#from sklearn_crfsuite import metrics

from seqeval.metrics import classification_report
from seqeval.scheme import IOB2
#import nlpnet

#from srl_bert_pt import tag_srl
import subprocess

import joblib

from readers.read_brat import ReadBrat
from readers.token_corpus import TokenCorpus


class TokSRL:
    def __init__(self):
        self.gov_verb = None
        self.arg = None
        self.tok = None

class EventModel:

    def __init__(self):

        current_path = os.path.dirname(__file__)
        srl_path = "srl_bert_pt/srl-enpt_xlmr-large/"
        full_srl_path = os.path.join(current_path, srl_path)

        subprocess.call("./srl_download.sh")
        #self.srl_tagger = tag_srl.TagSRL(full_srl_path)

        self.reader = ReadBrat()

        #self.srl_tagger = nlpnet.SRLTagger("srl-pt/",language="pt")
        # (name, transformer, columns)
        self.preprocessor = ColumnTransformer(
                transformers=[ 
                    ('token', TfidfVectorizer(),"token"), #TfidfVectorizer accepts column name only between quotes
                    ('lemma', TfidfVectorizer(),"lemma"), #TfidfVectorizer accepts column name only between quotes
                    ('head', TfidfVectorizer(),"head"), #TfidfVectorizer accepts column name only between quotes
                    ('head_lemma', TfidfVectorizer(),"head_lemma"), #TfidfVectorizer accepts column name only between quotes
                    ('gov_verb', TfidfVectorizer(),"gov_verb"), #TfidfVectorizer accepts column name only between quotes
                    ('category', OneHotEncoder(), ["head_pos","pos"]),
                    ],
                    remainder="drop"
                )


    def get_verb_pos(self, verb, tok_lst, idx_start):
        """
        It get the verb index in a given token list. The first occurrence,
        starting from idx_start index position

        @param string: the verb
        @param [string]: a list of tokens 
        @param int: the starting point to start the search for the verb

        @return int: the verb index in the token list
        """

        idx = idx_start
        while idx < len(tok_lst):
            if tok_lst[idx] == verb:
                break
            idx = idx + 1

        return idx

    def set_role_token(self, tokcorpus_lst, idx_verb, args):
        """
        It sets the role position for each token in the tokcorpus_lst list

        @param [TokenCorpus]: a list of TokenCorpus objects
        @param int: the index of the current governing verb of the given argument
                  structure
        @param dict: The dictionary that stores the argument structure of an SRL 
                    labeling returned by the nlpnet API

        @return [TokenCorpus]: the list of TokenCorpus objects updated with 
                    governing verb and srl role
        """

        # TODO 1: It is possible implement a more efficient method?
        # TODO 2: Perform a more complete test to see what examples does not work
        # here

        for role in args:
            for tok in args[role]:
                # talvez começar analise APOS idx_verb e, tokcorpus_lst!
                # nao pode, pois tem argumentos antes de idx_verb
                # Verbo mais proximo? Como saber qual eh o mais proximo?
                #  guardar posicao do gov_verb
                for idx_corpus, tokcorpus in enumerate(tokcorpus_lst):
                    if tok == tokcorpus.text:

                        # if still there is no role
                        if tokcorpus.srl is None:
                            tokcorpus.gov_verb = tokcorpus_lst[idx_verb].text
                            tokcorpus.gov_verb_idx = idx_verb
                            tokcorpus.srl = role
                            break
                        else:
                            # if there is role, but there is a more specific governing verb
                            # then assign a new role and a new governing verb
                            if tokcorpus.gov_verb_idx is not None:
                                # the nearest verb is more probable to be the gpverning verb
                                # so, this kind a heuristic according to the 
                                # nlpnet output
                                dist_verb = abs(idx_corpus - idx_verb)
                                dist_gov_verb = abs(idx_corpus - tokcorpus.gov_verb_idx)
                                if tokcorpus.gov_verb != tokcorpus_lst[idx_verb].text and\
                                        dist_verb <= dist_gov_verb:
                                    tokcorpus.gov_verb = tokcorpus_lst[idx_verb].text
                                    tokcorpus.srl = role

        #for tokcorpus in tokcorpus_lst:
        #    print(tokcorpus.text, tokcorpus.srl, tokcorpus.gov_verb)
        return tokcorpus_lst

    def get_srl_tags(self, doc):


        tok_lst = [tok.text for tok in doc]
        tags = self.srl_tagger.tag(tok_lst)

        all_toks_lst = []

        # tracking position of the verbs in the argument 
        # structures
        last_verb_idx = 0
        new_arg_lst = []
        for (verb, args) in tags.arg_structures:
            idx_verb = self.get_verb_pos(verb, tags.tokens, last_verb_idx)
            last_verb_idx = idx_verb
            new_arg_lst.append((verb, args, idx_verb))

        # mapping roles to tokens, using the governing verb positio
        tokcorpus_lst = [TokenCorpus(tok) for tok in tags.tokens]
        for verb, args, idx_verb in new_arg_lst:
            tokcorpus_lst = self.set_role_token(tokcorpus_lst, idx_verb, args)
        all_toks_lst = all_toks_lst + tokcorpus_lst
        return all_toks_lst

    def get_idx_token_srl(self, file_name):
        """
        Get the SRL tag of a given token

        @param string: the file to be tagged

        @return [(string, string, int)]: return a list of tuples that stores 
        (word, SRL tag, index in the text)
        """

        with open(file_name, "r") as fd:
            txt_ = fd.read()
            doc = self.reader.nlp(txt_)

            current_offset = 0
            token_tags_lst = []

            for sent in doc.sents:
                # get srl tags by sentence
                result = self.srl_tagger.tag(sent.text)

                # if there is verb in the sentence, 
                # then a tagged result is possible
                if "verbs" in result:
                    srl_tags = [(t, result["verbs"][0]["verb"]) for t in result["verbs"][0]["tags"]]
                    gov_verb = result["verbs"][0]["verb"]


                    # if there is more than one verb, these loops try to
                    # merge the srl tags
                    for i in range(1,len(result["verbs"])):

                        for j in range(len(result["verbs"][i]["tags"])):
                            if result["verbs"][i]["tags"][j] != "O":
                                tag = result["verbs"][i]["tags"][j]
                                gov_verb = result["verbs"][i]["verb"]
                                srl_tags[j] = (tag, gov_verb)

                    words = result["words"]
                else:
                    srl_tags = None
                    words = result
                    gov_verb = "None"

                # grouping the words and tags,
                # and in the process find in the text 
                # the character offset
                for idx, w in enumerate(words):
                    # character offset of the word w
                    offset_w = txt_.find(w, current_offset)
 
                    # updating global char offset
                    current_offset += len(w)

                    if srl_tags is not None:
                        token_tags_lst.append((w, srl_tags[idx], offset_w))
                    else:
                        token_tags_lst.append((w, ("O", gov_verb), offset_w))

        return token_tags_lst

    def get_srl_tags2(self, doc, file_name):
        """
        Since the srl tagger and the reader (Current ReadBrat) own different 
        tokenizers. This method present an heuristic to assign srl tags to doc.
        """

        token_tags_lst = self.get_idx_token_srl(file_name)

        i = 0

        while i < len(doc):
            # stores the last distance between the token from SRL
            # and the token from doc. So, it starts with None value, since
            # there is still no distance computed
            dist_old = None
            j = i
            while j < len(token_tags_lst):
                offset1 = token_tags_lst[j][2]
                offset2 = doc[i].offset

                label_srl , gov_verb = token_tags_lst[j][1]

                # the concept of distance is the difference between 
                # the offset of the token from the SRL tokenizer and
                # the offset of the token from the Reader tokenizer.
                # the smallets the diffrence, the nearest the words, and 
                # there is high probability that they are the same tokens.
                # So, the method assigns the SRL tag to the doc token.
                dist = abs(offset1 - offset2)
                if dist_old is None:
                    dist_old = dist
                    doc[i].srl = label_srl
                    doc[i].gov_verb = gov_verb
                else:
                    if dist < dist_old:
                        dist_old = dist
                        doc[i].srl = label_srl
                        doc[i].gov_verb = gov_verb
                j += 1
                
            i += 1


    def doc2feature(self, doc, file_name):
        """
        Extract the features of a text document for a CRF algorithm
        """

        data_doc = []
        y_doc = []

        #self.get_srl_tags2(doc, file_name)

        last_label = "O"
        for idx, tok in enumerate(doc):
            
            data_= {}
            # print(tok.text, "-",res2["words"][idx])

            data_["token"] = tok.text
            data_["lemma"] = tok.lemma
            data_["head"] = tok.head
            data_["head_lemma"] = tok.head_lemma
            data_["gov_verb"] = tok.gov_verb if tok.gov_verb is not None else "None"
            data_["srl"] = tok.srl if tok.srl is not None else "O"

            data_["head_pos"] = tok.head_pos
            data_["pos"] = tok.pos


            if tok.ann == "Event" and \
                (last_label == "I-EVENT" or last_label == "B-EVENT"):
                y_doc.append("I-EVENT")
                last_label = "I-EVENT"
            elif tok.ann == "Event" and last_label == "O":
                y_doc.append("B-EVENT")
                last_label = "B-EVENT"
            else:
                y_doc.append("O")
                last_label = "O"

            data_doc.append(data_)

        return data_doc, y_doc

    def _get_events_iob(self, doc, class_tokens):
        """
        Get a list of events from the labels of a iob labeling list
        """

        last_class = "O"
        count_event = 1
        current_event = {}
        event_lst = []

        symbols = string.punctuation + "\”\"\“"

        # get events

        for idx, tok in enumerate(doc):

            if class_tokens[idx] == "B-EVENT":

                current_event = {"id":count_event, "start":tok.offset, \
                        "end":tok.offset + len(tok.text),\
                        "value":tok.text}

                last_class = "B-EVENT"
                count_event += 1

                #print("T%d Event %d %d %s" % \
                #        (count_event, tok.offset, tok.offset + len(tok.text), tok.text))
            elif class_tokens[idx] == "I-EVENT":

                if last_class == "B-EVENT" or last_class == "I-EVENT":

                    id_ = current_event["id"]
                    start_ = current_event["start"]
                    end_ = tok.offset + len(tok.text)
                    if tok.text not in symbols:
                        value_ = current_event["value"] + " " + tok.text
                    else:
                        value_ = current_event["value"] + tok.text

                    current_event = {"id":id_,"start":start_,"end":end_, "value":value_}
                    last_class = "I-EVENT"
                else:
                    current_event = {"id":count_event, "start":tok.offset, \
                        "end":tok.offset + len(tok.text),\
                        "value":tok.text}

                    last_class = "I-EVENT"
                    count_event += 1
            else:
                if last_class == "B-EVENT" or last_class == "I-EVENT":
                    event_lst.append(current_event)
                    current_event = {}
                    last_class = "O"

        return event_lst

        

    def process_test(self, data_dir,model="crf",write_output=False):

        data_tokens = self.reader.process(data_dir)

        data_lst = []
        data_dct = {}
        y = []
        idx = 0

        for doc in data_tokens:
            data_doc, y_doc = self.doc2feature(doc, self.reader.file_lst[idx])
            data_lst.append(data_doc)
            y.append(y_doc)
            idx += 1

        clf = joblib.load("crf_v2.1.joblib")
        y_pred = clf.predict(data_lst)

        print(metrics.classification_report(flatten(y), flatten(y_pred)))


    def process(self, data_dir, model="crf"):
        """
        Read a data_dir (.ann and their .txt files)
        and returns a matrix of features, and labels

        @param string: data directory

        @return (numpy.array,numpy.array): Matrix of features and vector 
        of labels
        """

        # read file names .ann and .txt
        reader = ReadBrat()
        data_tokens = reader.process(data_dir)

        data_lst = []
        data_dct = {}
        y = []
        idx = 0
        for doc in data_tokens:
            data_doc, y_doc = self.doc2feature(doc, reader.file_lst[idx])
            data_lst.append(data_doc)
            y.append(y_doc)
            idx += 1


        clf = sklearn_crfsuite.CRF(algorithm='lbfgs',
                    c1=0.1, c2=0.1, max_iterations=100, 
                    all_possible_transitions=True)
        df_trans = data_lst

        clf.fit(df_trans, y)
        joblib.dump(clf, "crf_v2.1.joblib")

if __name__ == "__main__":

    data_dir = "data/train/"
    test_dir = "data/test/"
    model = EventModel()

    model.process(data_dir)
    model.process_test(test_dir)



"""
	text2story.core.annotator

	META-annotator
"""

from text2story.annotators import ACTOR_EXTRACTION_TOOLS, TIME_EXTRACTION_TOOLS, OBJECTAL_LINKS_RESOLUTION_TOOLS
from text2story.annotators import EVENT_EXTRACTION_TOOLS, SEMANTIC_ROLE_LABELLING_TOOLS
from text2story.annotators import extract_actors, extract_times, extract_objectal_links, extract_events
from text2story.annotators import extract_semantic_role_links


class Annotator:
    """
    Representation of a META-Annotator (a combination of one or more annotators).
    Useful to give a uniform interface to the rest of the package.

    Attributes
    ----------
    tools : list[str]
        the list of annotators to be used

    Methods
    -------
    extract_actors(lang, text)
        Returns a list with the actors identified in the text.
        Each actor is represented by a tuple, consisting of the start character offset, end character offset, the POS tag and the NE IOB tag, resp.
            Example: (0, 4, 'Noun', 'Per')
        Possible POS tags: 'Noun', 'Pronoun'.
        Possible NE IOB tags: 'Per', 'Org', 'Loc', 'Obj', 'Nat' and 'Other'.


    extract_times(lang, text)
        Returns a list with the times identified in the text.
        Each time is represented by a tuple, consisting of the start character offset, end character offset, it's type and it's value, resp.
            Example: (6, 17, 'DATE', '2021-08-31')

    extract_corefs(lang, text)
        Returns a list with the clusters of entities identified in the text
        Each cluster is a list with tuples, where every tuple is a 2D tuple with the start and end character offset of the span corresponding to the same entity.
            Example: [(0, 6), (20, 22)]
    """

    def __init__(self, tools):
        """
        Parameters
        ----------
        tools : list[str]
            the list of the annotators to be used; can be used any combination of them
            possible annotators are: 'spacy', 'nltk' and 'sparknlp'
        """
        self.tools = tools


    def extract_actors(self, lang, text):
        """
        Parameters
        ----------
        lang : str
            the language of the text
            current supported languages are: portuguese ('pt'); english ('en')
        text : str
            the text to be made the extraction

        Returns
        -------
        list[tuple[tuple[int, int], str, str]]
            the list of actors identified where each actor is represented by a tuple
        """

        # The extraction can be done with more than one tool.
        # Since different tools can make a different tokenization of the text, efforts were made to identify the same entity, even when character span doesn't match.
        # For instance, some tool identified the entity with character span (2, 7) and other (5, 10). We are assuming that the entirely entity has the char. span of (2, 10).
        # To do that, we are taking the first (in the sense of the char. span) identification made, and keep extending the character end offset as much as possible, with every entity that has a span that intersects with our current span.
        # Also, note that we are obtaning a bunch of POS tags and NE IOB tags and we just want one.
        # For the POS tag, we are taking the most common label.
        # For the NE IOB tag, we do the same, but we favor all labels versus the generic 'Other' label. That is, even if the label 'Other' is the most common, if we have a more specific one, we use that, instead.

        nr_tools = len(self.tools)

        # If no tool specified, use all
        if nr_tools == 0:
            self.tools = ACTOR_EXTRACTION_TOOLS
            nr_tools = len(self.tools)

        # Gather the annotations made by the tools specified and combine the results
        annotations = []
        idxs = []
        for tool in self.tools:
            annotations.append(extract_actors(tool, lang, text))

        final_annotation = []

        idxs = [0] * nr_tools # Current actor position from each tool


        while not(all([len(annotations[i]) == idxs[i] for i in range(nr_tools)])): # We finish when we consumed every actor identified by every tool
            tool_id = -1
            tool_id_start_char_offset = len(text) # len(self.text) acting as infinite

            # Get the next entity chunk to be gather (the one with the lowest start character span)
            for i in range(nr_tools):
                if idxs[i] < len(annotations[i]):

                    current_actor = annotations[i][idxs[i]]
                    current_actor_start_character_span = current_actor[0][0]

                    if current_actor_start_character_span < tool_id_start_char_offset:
                        tool_id = i
                        tool_id_start_char_offset = current_actor_start_character_span


            # For now, our actor consists of a unique annotation made by some tool
            actor_start_character_offset = annotations[tool_id][idxs[tool_id]][0][0]
            actor_end_character_offset   = annotations[tool_id][idxs[tool_id]][0][1]
            # For the lexical head and type we will accumulate the results and latter choose the best following a criterion
            actor_lexical_heads          = [annotations[tool_id][idxs[tool_id]][1]]
            actor_types                  = [annotations[tool_id][idxs[tool_id]][2]]

            idxs[tool_id] += 1 # Consume the annotation

            # Other tools may have identified the same actor.
            # We need to search if theres some intersection in the span of the identified actor, with the other tools.
            # That is, they identified the same actor, but maybe missed some initial part of it.
            # We identify this situation by looking to the character start char offset of the current actor identified by each tool,
            # and if it happens to be less than our end char offset of our current identified actor, then we can extend the current information we have.
            # Note that we may extend the end char offset, each force us to repete this process, till the offsets stabilize.

            # In the first interation, the tool that first identified the actor, will be matched and add double information and we don't want that
            # If we get to a second iteration, then that means the actor end char offset was extended which, in that case, the tool that first identified the actor, may now extend also...
            first_iteration = True
            while True:
                flag = False
                for i in range(nr_tools):
                    if first_iteration and i == tool_id:
                        continue

                    if idxs[i] < len(annotations[i]):
                        if annotations[i][idxs[i]][0][0] <= actor_end_character_offset:
                            if actor_end_character_offset < annotations[i][idxs[i]][0][1]:
                                actor_end_character_offset = annotations[i][idxs[i]][0][1]
                                flag = True

                            actor_lexical_heads.append(annotations[i][idxs[i]][1])
                            actor_types.append(annotations[i][idxs[i]][2])
                            idxs[i] = idxs[i] + 1

                first_iteration = False
                if not(flag):
                    break

            # Now that we identified the larger span possible for the actor, we need to fix the lexical head and type.

            # For the POS tag, we favor specifics the 'Noun' and 'Pronoun' tag and then we take the most common.
            # Since we only defined this labels, the others will appear as 'UNDEF'
            rmv_undef_pos_tags = [ne for ne in actor_lexical_heads if ne != 'UNDEF']
            if rmv_undef_pos_tags:
                actor_lexical_head = max(rmv_undef_pos_tags, key=rmv_undef_pos_tags.count)
            else:
                continue # Discard the actor if it's lexical head isn't a 'Noun' or 'Pronoun'

            # For the NE, we also favor specifics NEs, in this case all labels versus the NE 'OTHER' and we take the most common.
            rmv_other_ne = [ne for ne in actor_types if ne != 'Other']
            if rmv_other_ne:
                actor_type = max(rmv_other_ne, key=rmv_other_ne.count)
            else:
                actor_type = 'Other'

            # Discard entities with types other than 'Per', 'Org', 'Loc', 'Obj', 'Nat' & 'Other'.
            # Used, typically, to eliminate dates and durations incorrectly identified as an actor.
            if actor_type in ['Per', 'Org', 'Loc', 'Obj', 'Nat', 'Other']:
                final_annotation.append(((actor_start_character_offset, actor_end_character_offset), actor_lexical_head, actor_type))

        return final_annotation

    def extract_times(self, lang, text, publication_time):
        """
        Parameters
        ----------
        lang : str
            the language of the text
            current supported languages are: portuguese ('pt'); english ('en')
        text : str
            the text to be made the extraction
        publication_time: str
            the publication time

        Returns
        -------
        list[tuple[tuple[int, int], str, str]]
            a list consisting of the times identified, where each time is represented by a tuple
            with the start and end character offset, it's value and type, respectively
        """

        nr_tools = len(self.tools)

        # If no tool specified, use all
        if nr_tools == 0:
            self.tools = TIME_EXTRACTION_TOOLS
            nr_tools = len(self.tools)

        # NOTE: The extraction is done with only one tool, so the result in just the extraction done by the tool
        times = extract_times(self.tools[0], lang, text, publication_time) # :: [(time_start_offset, time_end_offset, time_type, time_value)]
        return times

    def extract_events(self, lang, text):
        """
        Event extraction. Only has one tool so it only returns what the annotator finds.

        @param lang: The language of the text
        @param text: The text to be annotated

        @return: Pandas DataFrame with each event found in the text and their character spans
        """
        nr_tools = len(self.tools)

        if nr_tools == 0:
            self.tools = EVENT_EXTRACTION_TOOLS
            nr_tools = len(self.tools)

        events = extract_events(self.tools[0], lang, text)
        return events

    def extract_objectal_links(self, lang, text):
        """
        Parameters
        ----------
        lang : str
            The language of the text.
            Current supported languages are: english ('en')
        text : str
            The text to be made the extraction.

        Returns
        -------
        list[list[tuple[int, int]]
            A list with the clusters identified.
            Each cluster is a list with tuples, where every tuple is a 2D tuple with the start and end character offset of the span corresponding to the same entity.
        """

        nr_tools = len(self.tools)

        # If no tool specified, use all
        if nr_tools == 0:
            self.tools = OBJECTAL_LINKS_RESOLUTION_TOOLS
            nr_tools = len(self.tools)

        # NOTE: The extraction is done with only one tool, so the result in just the extraction done by the tool
        return extract_objectal_links(self.tools[0], lang, text)

    def extract_semantic_role_links(self, lang, text):
        """
        Semantic Role Link extraction. Only has one tool, so no tool merging is needed.

        @param lang: The language of the text
        @param text: The text to be annotated

        @return: List of pandas DataFrames with the actors, events and their semantic roles to be linked
        """
        nr_tools = len(self.tools)

        # If no tool specified, use all
        if nr_tools == 0:
            self.tools = SEMANTIC_ROLE_LABELLING_TOOLS
            nr_tools = len(self.tools)

        srl_by_sentence = extract_semantic_role_links(self.tools[0], lang, text)
        return srl_by_sentence



'''
    PY_HEIDELTIME annotator (https://github.com/JMendes1995/py_heideltime)

    Used for:
        - Timexs extraction
            'en' : default
            'pt' : default
'''

#from text2story.core.exceptions import InvalidLanguage

from py_heideltime import py_heideltime
import re
import sys

def load(lang):
    """
    Used, at start, to load the pipeline for the supported languages.
    """
    pass # Nothing to load


def extract_times(lang, text, publication_time):
    """
    Parameters
    ----------
    lang : str
        the language of text to be annotated

    text : str
        the text to be annotated

    Returns
    -------
    list[tuple[tuple[int, int], str, str]]
        a list consisting of the times identified, where each time is represented by a tuple
        with the start and end character offset, it's value and type, respectively

    Raises
    ------
    InvalidLanguage if the language given is invalid/unsupported
    """

    if lang not in ['en', 'pt']:
        #raise InvalidLanguage
        print("Invalid Language")
        raise sys.exit(0)

    lang_mapping = {'pt' : 'Portuguese', 'en' : 'English'}
    lang = lang_mapping[lang]

    # annotations = py_heideltime(re.escape(text), language=lang, document_creation_time=publication_time)
    annotations = py_heideltime(text, language=lang, document_creation_time=publication_time)

    pattern = r'<TIMEX3 tid=.*? type=.*? value=.*?>.*?</TIMEX3>'

    timexs_list = re.finditer(pattern, annotations[2])

    timexs = []

    char_offset = 0

    for timex in timexs_list:
        match = timex.group()

        i = 13 # Discard all till the starting of the 'tid' value
        while match[i] != '"': # Consume the 'tid' value
            i += 1

        i += 8 # Now, we are at the field 'type'
        timex_type = ''
        while match[i] != '"':
            timex_type += match[i]
            i += 1

        i += 9 # Now, we are the field 'value'
        timex_value = ''
        while match[i] != '"':
            timex_value += match[i]
            i += 1

        # Now, we are the text of the timex
        while match[i] != '>': #ignore everything until it reaches the end of the tag
            i += 1
        i += 1

        timex_text = ''
        while match[i] != '<':
            timex_text += match[i]
            i += 1

        char_offset = text.find(timex_text, char_offset)

        timex_start_offset = char_offset
        timex_end_offset = char_offset + len(timex_text)
        timex_character_span =  (timex_start_offset, timex_end_offset)
        char_offset = timex_end_offset

        timex = (timex_character_span, timex_type, timex_value)
        timexs.append(timex)

    return timexs



"""
    A custom annotator for events in portuguese language

    The model is a random forest based on the lusa news labeled data
"""

# adding root directory
import sys

sys.path.insert(0, '..')
sys.path.insert(0, '../../')

import spacy
import joblib
import numpy as np
import pandas as pd

from text2story.readers.read_brat import ReadBrat

import os
# adding root directory
import sys
import string

sys.path.insert(0, '..')
sys.path.insert(0, '../../')

from text2story.readers.token_corpus import TokenCorpus

from sklearn.ensemble import GradientBoostingClassifier

pipeline = {}

# mapping of pos tag to a integer number
pos2idx = {"<S>": 1, "PROPN": 2, "PUNCT": 3, "NUM": 4, \
           "ADP": 5, "SPACE": 6, "DET": 7, "NOUN": 8, \
           "CCONJ": 9, "ADJ": 10, "VERB": 11, "ADV": 12, \
           "SCONJ": 13, "AUX": 14, "PRON": 15, "SYM": 16, \
           "X": 17}

# the size of the window to the
# features of the model, currently the model was trained with 2
WINDOW_SIZE = 2


def load(lang):

    if lang == "pt":
        current_path = os.path.dirname(__file__)
        model_name = "srl-pt_bertimbau-base"
        full_srl_path = os.path.join(current_path, "srl_bert_pt")
        full_srl_path = os.path.join(full_srl_path, model_name)


        pipeline["event_tagger"] = joblib.load(os.path.join(current_path, "crf_v2.1.joblib"))
        pipeline["pos_tagger"] = spacy.load("pt_core_news_lg")
    #pipeline["srl_tagger"] = tag_srl.TagSRL(full_srl_path)


def _get_data(idx_lst, all_ftrs):
    """
    it gets the data in all_ftr lst that has the index in the idx_lst

    @param [integer]: a list of index
    @param [[float]]: a list of lists of features
    """

    X = []
    y = []
    for i in idx_lst:
        for data in all_ftrs[i]:
            y.append(data[-1])

            # concat all the tokens in only one vector
            X.append(data[:-1])

    return np.asarray(X), y


def _get_pos_ftr(idx, doc_lst):
    """
    mapping of pos tagging to integer

    @param integer: the index of the token to get the pos tag code
    @param [(string,string)]: a list of tuples of the format (token, pos_tag)

    @return integer: a integer code that represents a pos tag
    """

    if idx < 0 or idx >= len(doc_lst):
        return pos2idx["<S>"]
    else:
        if len(doc_lst[idx]) > 2:
            (tok, pos, ann) = doc_lst[idx]
        else:
            (tok, pos) = doc_lst[idx]

        return pos2idx[pos]  # pos tag


def _extract_features_train(doc):
    """
    build features for each sentence tokenized by spacy

    @param document: a sentence as document type (spacy)

    @return np.array: an array of pos tagging features of a given text
    """
    ftrs = []
    idx = 0
    window = WINDOW_SIZE

    tok_lst = [(tok.text, tok.pos, tok.ann) for tok in doc]

    for (tok, pos, ann) in tok_lst:
        tok_ftrs = []

        for i in range(idx - window, idx + window + 1):
            tok_ftrs.append(_get_pos_ftr(i, tok_lst))

        if ann == 'Event':
            tok_ftrs.append("I")
        else:
            tok_ftrs.append("O")

        idx += 1
        ftrs.append(tok_ftrs)

    return np.array(ftrs)


def _extract_features(doc):
    """
    build features for each sentence tokenized by spacy

    @param document: a sentence as document type (spacy)

    @return np.array: an array of pos tagging features of a given text
    """
    ftrs = []
    idx = 0
    window = WINDOW_SIZE

    tok_lst = [(tok.text, tok.pos_) for tok in doc]

    for (tok, pos) in tok_lst:
        tok_ftrs = []

        for i in range(idx - window, idx + window + 1):
            tok_ftrs.append(_get_pos_ftr(i, tok_lst))

        idx += 1
        ftrs.append(tok_ftrs)

    return np.array(ftrs)


def execute_train(data_dir, reader, output_model="gb_custom_event_pt.joblib"):
    """
    Method to train a model to classify events, given a dataset

    @param string: path to the train data
    @param ReadBrat: a reader of the corpus
    @param string: the name of the podel to persist

    @return None
    """

    # read file names .ann and .txt
    reader = ReadBrat()
    data_tokens = reader.process(data_dir)

    data_lst = []
    data_dct = {}
    y = []
    idx = 0
    for doc in data_tokens:
        data_doc, y_doc = doc2featureCRF(doc, reader, file_name=reader.file_lst[idx])
        data_lst.append(data_doc)
        y.append(y_doc)
        idx += 1


    clf = sklearn_crfsuite.CRF(algorithm='lbfgs',
                c1=0.1, c2=0.1, max_iterations=100, 
                all_possible_transitions=True)
    df_trans = data_lst

    clf.fit(df_trans, y)
    joblib.dump(clf, output_model)


def _get_verb_pos(verb, tok_lst, idx_start):
    """
    It get the verb index in a given token list. The first occurrence,
    starting from idx_start index position

    @param string: the verb
    @param [string]: a list of tokens
    @param int: the starting point to start the search for the verb

    @return int: the verb index in the token list
    """

    idx = idx_start
    while idx < len(tok_lst):
        if tok_lst[idx] == verb:
            break
        idx = idx + 1

    return idx


def _set_role_token(tokcorpus_lst, idx_verb, args):
    """
    It sets the role position for each token in the tokcorpus_lst list

    @param [TokenCorpus]: a list of TokenCorpus objects
    @param int: the index of the current governing verb of the given argument
              structure
    @param dict: The dictionary that stores the argument structure of an SRL
                labeling returned by the nlpnet API

    @return [TokenCorpus]: the list of TokenCorpus objects updated with
                governing verb and srl role
    """

    # TODO 1: It is possible implement a more efficient method?
    # TODO 2: Perform a more complete test to see what examples does not work
    # here

    for role in args:
        for tok in args[role]:
            # talvez começar analise APOS idx_verb e, tokcorpus_lst!
            # nao pode, pois tem argumentos antes de idx_verb
            # Verbo mais proximo? Como saber qual eh o mais proximo?
            #  guardar posicao do gov_verb
            for idx_corpus, tokcorpus in enumerate(tokcorpus_lst):
                if tok == tokcorpus.text:

                    # if still there is no role
                    if tokcorpus.srl is None:
                        tokcorpus.gov_verb = tokcorpus_lst[idx_verb].text
                        tokcorpus.gov_verb_idx = idx_verb
                        tokcorpus.srl = role
                        break
                    else:
                        # if there is role, but there is a more specific governing verb
                        # then assign a new role and a new governing verb
                        if tokcorpus.gov_verb_idx is not None:
                            # the nearest verb is more probable to be the gpverning verb
                            # so, this kind a heuristic according to the
                            # nlpnet output
                            dist_verb = abs(idx_corpus - idx_verb)
                            dist_gov_verb = abs(idx_corpus - tokcorpus.gov_verb_idx)
                            if tokcorpus.gov_verb != tokcorpus_lst[idx_verb].text and \
                                    dist_verb <= dist_gov_verb:
                                tokcorpus.gov_verb = tokcorpus_lst[idx_verb].text
                                tokcorpus.srl = role

    # for tokcorpus in tokcorpus_lst:
    #    print(tokcorpus.text, tokcorpus.srl, tokcorpus.gov_verb)
    return tokcorpus_lst


def get_idx_token_srl(reader, txt_):
    """
    Get the SRL tag of a given token

    @param string: the file to be tagged

    @return [(string, string, int)]: return a list of tuples that stores 
    (word, SRL tag, index in the text)
    """

    doc = reader.nlp(txt_)

    current_offset = 0
    token_tags_lst = []

    for sent in doc.sents:
        # get srl tags by sentence
        result = pipeline["srl_tagger"].tag(sent.text)

        # if there is verb in the sentence, 
        # then a tagged result is possible
        if "verbs" in result:
            srl_tags = [(t, result["verbs"][0]["verb"]) for t in result["verbs"][0]["tags"]]
            gov_verb = result["verbs"][0]["verb"]


            # if there is more than one verb, these loops try to
            # merge the srl tags
            for i in range(1,len(result["verbs"])):

                for j in range(len(result["verbs"][i]["tags"])):
                    if result["verbs"][i]["tags"][j] != "O":
                        tag = result["verbs"][i]["tags"][j]
                        gov_verb = result["verbs"][i]["verb"]
                        srl_tags[j] = (tag, gov_verb)

            words = result["words"]
        else:
            srl_tags = None
            words = result
            gov_verb = "None"

        # grouping the words and tags,
        # and in the process find in the text 
        # the character offset
        for idx, w in enumerate(words):
            # character offset of the word w
            offset_w = txt_.find(w, current_offset)

            # updating global char offset
            current_offset += len(w)

            if srl_tags is not None:
                token_tags_lst.append((w, srl_tags[idx], offset_w))
            else:
                token_tags_lst.append((w, ("O", gov_verb), offset_w))

    return token_tags_lst

def set_srl_tags(doc, reader, txt):
    """
    Since the srl tagger and the reader (Current ReadBrat) own different 
    tokenizers. This method present an heuristic to assign srl tags to doc.
    """

    token_tags_lst = get_idx_token_srl(reader, txt)

    i = 0

    while i < len(doc):
        # stores the last distance between the token from SRL
        # and the token from doc. So, it starts with None value, since
        # there is still no distance computed
        dist_old = None
        j = i
        while j < len(token_tags_lst):
            offset1 = token_tags_lst[j][2]
            offset2 = doc[i].offset

            label_srl , gov_verb = token_tags_lst[j][1]

            # the concept of distance is the difference between 
            # the offset of the token from the SRL tokenizer and
            # the offset of the token from the Reader tokenizer.
            # the smallets the diffrence, the nearest the words, and 
            # there is high probability that they are the same tokens.
            # So, the method assigns the SRL tag to the doc token.
            dist = abs(offset1 - offset2)
            if dist_old is None:
                dist_old = dist
                doc[i].srl = label_srl
                doc[i].gov_verb = gov_verb
            else:
                if dist < dist_old:
                    dist_old = dist
                    doc[i].srl = label_srl
                    doc[i].gov_verb = gov_verb
            j += 1
            
        i += 1

def doc2featureCRF(doc, reader, file_name=None, text=None):

    data_doc = []
    y_doc = []

    if text is None:
        if file_name is None:
            raise Exception("CustomPT Model: Unknown source of the text. \
                    Please, provide a raw text for labeling.")
        else:
            with open(file_name, "r") as fd:
                text = fd.read()

    # TODO: SRL portuguese requires a strong hardware
    #set_srl_tags(doc, reader, text)

    idx = 0
    last_label = "O"
    sent = []

    for idx, tok in enumerate(doc):
        
        data_= {}
        # print(tok.text, "-",res2["words"][idx])

        data_["token"] = tok.text
        data_["head"] = tok.head.text

        # if it is Token object
        if hasattr(tok, "head_lemma"):
            data_["lemma"] = tok.lemma_
            data_["head_lemma"] = tok.head_lemma
            data_["head_pos"] = tok.head_pos
            data_["gov_verb"] = tok.gov_verb if tok.gov_verb is not None else "None"
            data_["srl"] = tok.srl if tok.srl is not None else "O"
            data_["pos"] = tok.pos
        else:
            # else it is a Spacy.Token object
            data_["lemma"] = tok.lemma_
            data_["head_lemma"] = tok.head.lemma_
            data_["head_pos"] = tok.head.pos_
            data_["gov_verb"] = "None"
            data_["srl"] = tok.dep_
            data_["pos"] = tok.pos_


        if hasattr(tok, "ann"):
            if tok.ann == "Event" and \
                    (last_label == "I-EVENT" or last_label == "B-EVENT"):
                y_doc.append("I-EVENT")
                last_label = "I-EVENT"
            elif tok.ann == "Event" and last_label == "O":
                y_doc.append("B-EVENT")
                last_label = "B-EVENT"
            else:
                y_doc.append("O")
                last_label = "O"

        if hasattr(tok,"is_sent_start"):
            if tok.is_sent_start:
                if sent!= []:
                    data_doc.append(sent)
                sent = [data_]
            else:
                sent.append(data_)
        else:
            data_doc.append(data_)

    if sent != []:
        data_doc.append(sent)

    return data_doc, y_doc



def extract_events(lang, text):
    """
    Main function that applies the custom tagger to extract event entities from each sentence.

    @param lang: The language of the text
    @param text: The full text to be annotated

    @return: Pandas DataFrame with every event entity and their character span
    """
    # 1. use spacy to tokenize and to get pos tags
    doc = pipeline["pos_tagger"]((text))

    reader = ReadBrat()
    # FIND EVENTS#
    ftrs, _ = doc2featureCRF(doc,reader, text=text)

    
    clf = pipeline["event_tagger"]

    y = clf.predict(ftrs)

    result = {"actor": [], "char_span": []}
    symbols = string.punctuation + "\”\"\“"

    # colocar em outro formato aqui
    idx_sent = 0
    for sent in doc.sents:
        idx_tok = 0
        for tok in sent:
            if y[idx_sent][idx_tok] == "I-EVENT" or y[idx_sent][idx_tok] == "B-EVENT":
                # TODO: the model with punctuation seems to be 
                # better, but it includes punctuation as events. 
                # So, fix this. However, try to fix this in another way.
                if tok.text not in symbols:
                    result["actor"].append(tok.text)
                    result["char_span"].append((tok.idx, tok.idx + len(tok.text)))
            idx_tok += 1
        idx_sent += 1

    return pd.DataFrame(result)


if __name__ == "__main__":
    load()
    data_dir = "data/train"
    reader = ReadBrat()
    execute_train(data_dir, reader)
    # unit test
    # ans = extract_events("pt","O cavalo correu na pista de corrida. E tudo deu certo naquele dia. Depois o cavalo assaltou um casal.")
    # print(ans)




# gather corpus stats
import os
import sys

from pathlib import Path
from text2story.readers.read_brat import  ReadBrat
from nltk import word_tokenize


def count_tokens(fd):
    return len(word_tokenize(fd.read()))

def count_elements(fd, element_type):

    count = 0
    for line in fd:
        if line[0] == "T":
            elements = line.split()
            if elements[1] == element_type:
                count += 1

    return count

def build_stats_byelement(data_dir, element_type="event", dataset_type="BRAT"):
    """

    Cou

    @param data_dir: A string of data directory
    @param dataset_type: default type assumes the BRAT standoff format
    @return: dict of {attribute:frequency}
    """
    pass

def build_stats(data_dir):


    counts = {"tokens":[],"events":[],"participants":[],"time":[]}

    for dirpath, dirnames, filenames in os.walk(data_dir):
        for f in filenames:
            if f.endswith(".ann"):

                full_name = os.path.join(dirpath, f)
                p = Path(f)
                txt_file = os.path.join(data_dir,p.stem + ".txt")

                with open(txt_file, "r") as fd:
                    counts["tokens"].append(count_tokens(fd))

                with open(full_name, "r") as fd:
                    counts["events"].append(count_elements(fd, "Event"))

                with open(full_name, "r") as fd:

                    counts["participants"].append(count_elements(fd, "Participant"))
                with open(full_name, "r") as fd:
                    counts["time"].append(count_elements(fd, "Time"))

    print("Total #tokens:", sum(counts["tokens"]))
    print("Average #tokens:", sum(counts["tokens"])/len(counts["tokens"]))
    print("Total #events:", sum(counts["events"]))
    print("Total #participants:", sum(counts["participants"]))
    print("Total #time:", sum(counts["time"]))

if __name__ == "__main__":
    data_dir = sys.argv[1]
    build_stats(data_dir)



"""
This is a script to generate visualization of
brat files in a batch way.
"""
import plantuml

import argparse
import os

import text2story.brat2viz.drs2viz.parser as parser
from text2story.brat2viz.brat2drs import brat2drs

def dir_path(path):
    if os.path.isdir(path):
        return path
    else:
        raise argparse.ArgumentTypeError(f"readable_dir:{path} is not a valid path")

def list_ann_files(inputdir):
    """
    It collects the names of .ann files in the inputdir directory
    """

    lst_files = []
    for dirpath, dirnames, filenames in os.walk(inputdir):
        for f in filenames:
            if f.endswith(".ann"):
                lst_files.append(os.path.join(dirpath,f))

    return lst_files

def build_drs(file_name_ann, file_name_drs):

    filecontent = brat2drs.read_file(file_name_ann)
    f_parser = brat2drs.file_parser(filecontent)
    dexpr_list, f_parser = brat2drs.assign_variable(f_parser)
    dr_set, dexpr_list = brat2drs.attributes_events(dexpr_list, f_parser)
    dexpr_list = brat2drs.event_event_relation(f_parser, dexpr_list)

    actors, actors_sr = brat2drs.actors_relation(f_parser)

    relations, dexpr_list = brat2drs.TE_relations(f_parser, dexpr_list)
    relations, dexpr_list = brat2drs.TT_relations(relations, f_parser, dexpr_list)

    actors = brat2drs.update_actors(relations, actors, f_parser)

    brat2drs.write_output(dexpr_list, dr_set, actors, relations, file_name_drs)

def create_plantuml(actors_dict, events_dict, events_relations, non_event_relations):

    msc_str = ""
    for x in non_event_relations:
        ref1 = x[0]
        ref2 = x[2]
        rel_type = x[1]
        msc_str = msc_str + "\"%s\"->\"%s\":%s\n" % (actors_dict[ref1], actors_dict[ref2], rel_type)

    for e in events_relations:
        for (ref1,rel_type,ref2) in events_relations[e]:
            participant1 = actors_dict[ref1]
            participant2 = actors_dict[ref2]
            msc_str = msc_str + "\"%s\"->\"%s\":%s (%s)\n" % (participant1, participant2, events_dict[e],rel_type)
        #print("-->",e,events_relations[e])

    return msc_str

def drs2vis(drs_file):
    # url server to generate the figures
    url = "http://www.plantuml.com/plantuml/img/"
    pl = plantuml.PlantUML(url)

    file_name_out = os.path.basename(drs_file)
    file_name_out = os.path.splitext(drs_file)[0]

    # get entities
    actors_dict, events_dict, events_relations, non_event_relations = \
            parser.parse_drs(drs_file)

    # build plantuml txt file
    msc_str = create_plantuml(actors_dict, events_dict, \
                                events_relations, non_event_relations)

    with open(file_name_out,"w") as fd:
        fd.write(msc_str)

    file_name_fig = "%s.png" % (file_name_out)

    success = pl.processes_file(file_name_out, outfile=file_name_fig)

def msc_vis(lst_files, outputdir):

    # url server to generate the figures
    url = "http://www.plantuml.com/plantuml/img/"
    pl = plantuml.PlantUML(url)

    for f in lst_files:

        file_name_out = os.path.basename(f)
        file_name_out = os.path.splitext(file_name_out)[0]

        # build drs
        file_name_drs = os.path.join(outputdir, "%s.drs" % (file_name_out))
        build_drs(f, file_name_drs)

        # get entities
        actors_dict, events_dict, events_relations, non_event_relations = \
                parser.parse_drs(file_name_drs)

        # build plantuml txt file
        msc_str = create_plantuml(actors_dict, events_dict, \
                                    events_relations, non_event_relations)
        #print(msc_str)
        file_name_plantuml = os.path.join(outputdir,"%s.txt" % (file_name_out))
        with open(file_name_plantuml,"w") as fd:
            fd.write(msc_str)

        file_name_fig = os.path.join(outputdir, "%s.png" % (file_name_out))

        success = pl.processes_file(file_name_plantuml, outfile=file_name_fig)

#if __name__ == "__main__":

#    my_parser = argparse.ArgumentParser(description='This is a script to generate visualization of\
#brat files in a batch way.')

#    my_parser.add_argument("inputdir", action='store', type=dir_path, help="The directory that \
#            contains the target files (brat format) and the txt narrative files.")
#    my_parser.add_argument("outputdir", action='store', type=dir_path, help="The directory that \
#            will be written the MSC files.")

    # TODO: visualization in graph format

#    args = my_parser.parse_args()

#    ann_files = list_ann_files(args.inputdir)
#    msc_vis(ann_files, args.outputdir)



"""
	text2story.core.link_structures

	Link structure classes (Temporal, aspectual, subordination, semantic role and objectal)
"""


class TemporalLink:
    pass


class AspectualLink:
    pass


class SubordinationLink:
    pass


class SemanticRoleLink:
    def __init__(self, actor, event, type="theme"):
        self.type = type
        self.actor = actor
        self.event = event


class ObjectalLink:
    def __init__(self, arg1, arg2):
        self.type = 'objIdentity'
        self.arg1 = arg1
        self.arg2 = arg2



#https://stackoverflow.com/questions/18599339/watchdog-monitoring-file-for-changes
import time
from datetime import datetime, timedelta
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import brat2drs

ANN_PATH = 'text_2_story_new/'

class MyHandler(FileSystemEventHandler):
    def __init__(self):
        self.last_modified = datetime.now()

    def on_modified(self, event):
        if datetime.now() - self.last_modified < timedelta(seconds=1):
            return
        else:
            self.last_modified = datetime.now()
        print(f'Event type: {event.event_type}  path : {event.src_path}')
        brat2drs.main(ANN_PATH)
        #print(event.is_directory) # This attribute is also available


if __name__ == "__main__":
    event_handler = MyHandler()
    observer = Observer()
    observer.schedule(event_handler, path=ANN_PATH, recursive=False)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()
    


'''
    SPARKNLP annotator

    Used for actor extraction
        'pt' : wikiner_6B_100
        'en' : default
'''

from text2story.core.utils import chunknize_actors
from text2story.core.exceptions import InvalidLanguage

import sparknlp
from pyspark.sql import SparkSession
from sparknlp.base import DocumentAssembler, LightPipeline
from sparknlp.annotator import Tokenizer, PerceptronModel, WordEmbeddingsModel, NerDLModel, NerCrfModel, SentenceDetector, RoBertaForTokenClassification
from pyspark.ml import Pipeline
import pandas as pd

import os

pipeline = {}

def load_embeddings():

    current_path = os.path.dirname(os.path.abspath(__file__))
    #embeddings_path = os.path.join("cache","glove_100d")
    embeddings_path = os.path.join("cache","glove_840B_300")
    embeddings_path = os.path.join(current_path, embeddings_path)

    embeddings = None

    if os.path.exists(embeddings_path):
        # cache the pretrained model to speed up local computations
        print("Loading local model of glove_840B_300")
        embeddings = WordEmbeddingsModel.load(embeddings_path)
    else:
        # downloading and saving the pre-trained model

        # embeddings = WordEmbeddingsModel.pretrained('glove_100d').setInputCols(["token", "document"]).setOutputCol("embeddings")
        embeddings = WordEmbeddingsModel.pretrained("glove_840B_300", "xx").setInputCols(["document","token"]).setOutputCol("embeddings")

        if not(os.path.exists(os.path.join(current_path,"cache"))):
            os.mkdir(os.path.join(current_path, "cache"))

        print("Saving %s locally..." % embeddings_path)
        embeddings.save(embeddings_path)

    return embeddings

def load_postagger_pt():

    current_path = os.path.dirname(os.path.abspath(__file__))
    postagger_path = os.path.join("cache","pos_ud_bosque")
    postagger_path = os.path.join(current_path, postagger_path)

    postagger = None

    if os.path.exists(postagger_path):
        # cache the pretrained model to speed up local computations
        print("Loading local model of pos_ud_bosque")
        postagger = PerceptronModel.load(postagger_path)
    else:
        # downloading and saving the pre-trained model

        postagger     = PerceptronModel.pretrained('pos_ud_bosque', 'pt').setInputCols(["document","token"]).setOutputCol("pos")

        if not(os.path.exists(os.path.join(current_path,"cache"))):
            os.mkdir(os.path.join(current_path, "cache"))

        print("Saving %s locally..." % postagger_path)
        postagger.save(postagger_path)

    return postagger

def load_postagger_en():

    current_path = os.path.dirname(os.path.abspath(__file__))
    postagger_path = os.path.join("cache","pos_anc")
    postagger_path = os.path.join(current_path, postagger_path)

    postagger = None

    if os.path.exists(postagger_path):
        # cache the pretrained model to speed up local computations
        print("Loading local model of pos_anc")
        postagger = PerceptronModel.load(postagger_path)
    else:
        # downloading and saving the pre-trained model

        postagger     = PerceptronModel.pretrained('pos_anc', 'en').setInputCols(["token", "document"]).setOutputCol("pos")

        if not(os.path.exists(os.path.join(current_path,"cache"))):
            os.mkdir(os.path.join(current_path, "cache"))

        print("Saving %s locally..." % postagger_path)
        postagger.save(postagger_path)

    return postagger

def load_ner_en():

    current_path = os.path.dirname(os.path.abspath(__file__))
    ner_path = os.path.join("cache","neren")
    ner_path = os.path.join(current_path, ner_path)

    postagger = None

    if os.path.exists(ner_path):
        # cache the pretrained model to speed up local computations
        print("Loading local model of NER en")
        ner = NerCrfModel.load(ner_path)
    else:
        # downloading and saving the pre-trained model

        ner      = NerCrfModel.pretrained().setInputCols(["document", "token", "pos", "embeddings"]).setOutputCol("ner")

        if not(os.path.exists(os.path.join(current_path,"cache"))):
            os.mkdir(os.path.join(current_path, "cache"))

        print("Saving %s locally..." % ner_path)
        ner.save(ner_path)

    return ner

def load_ner_pt():

    current_path = os.path.dirname(os.path.abspath(__file__))
    #ner_path = os.path.join("cache","wikiner")
    #ner_path = os.path.join("cache","roberta_ner_pt")
    ner_path = os.path.join("cache","wikiner_840B_300")
    ner_path = os.path.join(current_path, ner_path)

    postagger = None

    if os.path.exists(ner_path):
        # cache the pretrained model to speed up local computations
        print("Loading local model of Roberta NER PT")
        ner = NerDLModel.load(ner_path)
        #ner = NerCrfModel.load(ner_path)
        #ner = RoBertaForTokenClassification.load(ner_path)
    else:
        # downloading and saving the pre-trained model

        ner = NerDLModel.pretrained("wikiner_840B_300", 'pt').setInputCols(['document', 'token', 'embeddings']).setOutputCol('ner')

        #ner = RoBertaForTokenClassification.pretrained("wikiner_840B_300","pt").setInputCols(["sentence", "token"]).setOutputCol("ner")

        #ner      = NerDLModel.pretrained('wikiner_6B_100', 'pt').setInputCols(["document", "token", "embeddings"]).setOutputCol("ner")

        if not(os.path.exists(os.path.join(current_path,"cache"))):
            os.mkdir(os.path.join(current_path, "cache"))

        print("Saving %s locally..." % ner_path)
        ner.save(ner_path)

    return ner

def load(lang):
    """
    Used, at start, to load the pipeline for the supported languages.
    """

    #sparknlp.start(spark32=True)
    sparknlp.start()
    spark = SparkSession.builder.appName("t2s").getOrCreate()
    spark.sparkContext.setLogLevel("FATAL")

    documentAssembler = DocumentAssembler().setInputCol("text").setOutputCol("document")

    sentence_detector = SentenceDetector().setInputCols(["document"]).setOutputCol("sentence")

    tokenizer         = Tokenizer().setInputCols(["sentence"]).setOutputCol("token")

    # TODO: esse embeddings sao em ingles, usar em pt. Por isso que deve estar dando aquele erro
    # para portugues!
    embeddings = load_embeddings()

    if lang == "pt":
        pos_tagger_pt     = load_postagger_pt()
        ner_model_pt      = load_ner_pt()
        pipeline_pt = Pipeline(stages=[documentAssembler, sentence_detector,tokenizer, embeddings, pos_tagger_pt, ner_model_pt])
        pipeline['pt'] = LightPipeline(pipeline_pt.fit(spark.createDataFrame(pd.DataFrame({'text': ['']}))))
    elif lang == "en":
        pos_tagger_en     = load_postagger_en()
        ner_model_en      = load_ner_en()
        pipeline_en = Pipeline(stages=[documentAssembler, sentence_detector, tokenizer, embeddings, pos_tagger_en, ner_model_en])
        pipeline['en'] = LightPipeline(pipeline_en.fit(spark.createDataFrame(pd.DataFrame({'text': ['']}))))
    else:
        raise InvalidLanguage(lang)


def extract_actors(lang, text):
    """
    Parameters
    ----------
    lang : str
        the language of text to be annotated
    text : str
        the text to be annotated
    
    Returns
    -------
    list[tuple[tuple[int, int], str, str]]
        the list of actors identified where each actor is represented by a tuple

    Raises
    ------
        InvalidLanguage if the language given is invalid/unsupported
    """

    if lang not in ['pt', 'en']:
        raise InvalidLanguage

    doc = pipeline[lang].fullAnnotate(text)[0] 

    iob_token_list = []
    for i in range(len(doc['token'])):
        start_char_offset = doc['token'][i].begin
        end_char_offset   = doc['token'][i].end + 1
        char_span         = (start_char_offset, end_char_offset)
        pos_tag           = normalize(doc['pos'][i].result)
        ne                = doc['ner'][i].result[:2] + normalize(doc['ner'][i].result[2:]) if doc['ner'][i].result[:2] != 'O' else 'O'

        iob_token_list.append((char_span, pos_tag, ne))

    actor_list = chunknize_actors(iob_token_list)

    return actor_list


def normalize(label):
    """
    Parameters
    ----------
    label : str
    
    Returns
    -------
    str
        the label normalized
    """

    mapping = {
        # POS tags 
        # en: (Penn Treebank Project: https://www.ling.upenn.edu/courses/Fall_2003/ling001/penn_treebank_pos.html)
        'NN'    : 'Noun',
        'NNS'   : 'Noun',
        'NNP'   : 'Noun',
        'NNPS'  : 'Noun',
        'PRP'   : 'Pronoun',
        'PRP$'  : 'Pronoun',
        'WP'    : 'Pronoun',
        'WP$'   : 'Pronoun',
    
        # pt: Universal POS Tags: http://universaldependencies.org/u/pos/
        #"ADJ": "adjective",
        #"ADP": "adposition",
        #"ADV": "adverb",
        #"AUX": "auxiliary",
        #"CONJ": "conjunction",
        #"CCONJ": "coordinating conjunction",
        #"DET": "determiner",
        #"INTJ": "interjection",
        "NOUN": "Noun",
        #"NUM": "numeral",
        #"PART": "particle",
        "PRON": "Pronoun",
        "PROPN": "Noun",
        #"PUNCT": "punctuation",
        #"SCONJ": "subordinating conjunction",
        #"SYM": "symbol",
        #"VERB": "verb",
        #"X": "other",
        #"EOL": "end of line",
        #"SPACE": "space",

        # NE labels
        'LOC'   : 'Loc',
        'ORG'   : 'Org',
        'PER'   : 'Per',
        'MISC'  : 'Other'    
    }

    return mapping.get(label, 'UNDEF')



import os
import json
from flask import Flask, render_template, request, url_for
import parser as parser

app = Flask(__name__)

#BRAT_ANN_DIR = '../brat2drs/text_2_story_new'
BRAT_ANN_DIR = '../brat2drs/sample/'


@app.route('/brat2viz', methods=['GET', 'POST'])
def brat2viz():
    drs_files = parser.get_drs_files()

    selected_drs = request.form.get('select_drs')
    # nunca esta pegando nada aqui em select drs, pq?

    if selected_drs:
        drs_file = selected_drs
    else:
        drs_file = drs_files[0]

    selected_vis = request.form.get('select_vis')

    common_file_name = drs_file.split('/')[-1].split('_drs')[0]
    #print(selected_vis,selected_drs)

    if selected_vis == 'ann_text':
        vis = 'ann_text'
        ann_file = os.path.join(BRAT_ANN_DIR, common_file_name + '.ann')
        with open(ann_file, 'r') as fp:
            ann_text = fp.readlines()
        return render_template('index.html', drs_files=drs_files, selected_drs=drs_file, selected_vis=vis,
                               text=ann_text)

    elif selected_vis == 'drs_text':
        vis = 'drs_text'
        with open(drs_file, 'r') as fp:
            drs_text = fp.readlines()
        return render_template('index.html', drs_files=drs_files, selected_drs=drs_file, selected_vis=vis,
                               text=drs_text)

    elif selected_vis == 'msc':
        vis = 'msc'
        msc = parser.get_msc_data(drs_file)
        return render_template('index.html', drs_files=drs_files, selected_drs=drs_file, selected_vis=vis, msc=msc)

    elif selected_vis == 'graph':
        vis = 'graph'
        actors, non_ev_rels, ev_rels = parser.get_graph_data(drs_file)
        actors = json.dumps(actors)
        non_ev_rels = json.dumps(non_ev_rels)
        ev_rels = json.dumps(ev_rels)
        return render_template('index.html', drs_files=drs_files, selected_drs=drs_file, selected_vis=vis,
                               actors=actors, non_ev_rels=non_ev_rels, ev_rels=ev_rels)

    # Default
    else:
        vis = 'news_text'
        news_file = os.path.join(BRAT_ANN_DIR, common_file_name + '.txt')
        with open(news_file, 'r') as fp:
            news_text = fp.readlines()
        return render_template('index.html', drs_files=drs_files, selected_drs=drs_file, selected_vis=vis,
                               text=news_text)


if __name__ == "__main__":
    app.run(host='0.0.0.0', port=5055, debug=True)



"""
Weak labeling performed using SRL and Neural Models

For the Participant extraction the training method is in the Class training/participant_concept.ParticipantsModel

@author: Evelin Amorim
"""
import torch
import os
import gdown

from transformers import BertTokenizer, BertModel

from text2story.annotators import ALLENNLP
from text2story.core.exceptions import InvalidLanguage

from text2story.training.participant_concept import  ParticipantsModel

from nltk import sent_tokenize

pipeline = {}
device = "cpu"
max_seq_len = 128
threshold = 0.5


def download_participant_pt_model(participant_pt_model):
    print("Downloading %s. It can take some time..." % participant_pt_model)

    url = "https://drive.google.com/uc?id=1mnaTfOcKyoKFclhA430VqvIsheLMeLdB&export=download"
    gdown.download(url, participant_pt_model, quiet=False)

def load_pt():
    ALLENNLP.load("pt")

    current_path = os.path.dirname(os.path.abspath(__file__))

    participant_pt_model_path = os.path.join(current_path, "cache")
    if not (os.path.exists(participant_pt_model_path)):
        os.mkdir(participant_pt_model_path)

    participant_pt_model = os.path.join(participant_pt_model_path, "participant_saved_weights.pt")
    pipeline["participants_model_file"] = participant_pt_model

    if not (os.path.exists(participant_pt_model)):
        download_participant_pt_model(participant_pt_model)

    pipeline["tokenizer"] = BertTokenizer.from_pretrained('neuralmind/bert-base-portuguese-cased', \
                                                         model_max_length=128)
    pipeline["bert_model"] = BertModel.from_pretrained('neuralmind/bert-base-portuguese-cased')

    path_loader = torch.load(participant_pt_model, map_location=torch.device(device))
    participant_model = ParticipantsModel(pipeline["bert_model"], 128)
    participant_model.load_state_dict(path_loader)
    pipeline["participants_model"] = participant_model

    #pipeline["tag"] = model.get("id_map")


def load(lang):
    if lang == "pt":
        load_pt()
    else:
        raise InvalidLanguage


def extract_actors(lang, text):

    sent_lst = sent_tokenize(text)

    for sent in sent_lst:
        print(sent)
        tokens = pipeline["tokenizer"].tokenize(sent)

        tokens = tokens[:max_seq_len - 2]
        tokens = ['[CLS]'] + tokens + ['[SEP]']

        input_ids = pipeline["tokenizer"].convert_tokens_to_ids(tokens)
        input_ids = input_ids + [0] * (max_seq_len - len(input_ids))
        input_ids = torch.tensor(input_ids).unsqueeze(0)
        input_ids = input_ids.to(device)

        # extrair srl da sentenca e converter para input ids tbm
        candidate_lst = ALLENNLP.extract_actors(lang, sent)

        # cada span tem uma sentenca, entao tenho que reproduzir isso!
        span_text = []
        for ((start,end), _, _) in candidate_lst:
            span_text = sent[start:end]
            tokens_span = pipeline["tokenizer"].tokenize(span_text)

            input_ids_span = pipeline["tokenizer"].convert_tokens_to_ids(tokens_span)
            input_ids_span = input_ids_span + [0] * (max_seq_len - len(input_ids_span))
            input_ids_span = torch.tensor(input_ids_span).unsqueeze(0)
            input_ids_span = input_ids.to(device)

            preds = pipeline["participants_model"](input_ids, input_ids_span)
            preds = (preds >= threshold).int()

            print(preds)
            print("SPAN TEXT", span_text)
            #print(len(preds), len())

            print()
